# -*- coding: utf-8 -*-
from design.plone.contenttypes import _
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


@provider(IFormFieldProvider)
class IExcludeFromSearch(model.Schema):
    """ """

    exclude_from_search = schema.Bool(
        title=_("exclude_from_search_label", default="Escludi dalla ricerca"),
        description=_(
            "help_exclude_from_search",
            default="Se selezionato, questo contenuto non verrà mostrato nelle ricerche del sito per gli utenti anonimi.",
        ),
        required=False,
        default=False,
    )
    model.fieldset(
        "settings",
        fields=["exclude_from_search"],
    )


@implementer(IExcludeFromSearch)
@adapter(IDexterityContent)
class ExcludeFromSearch(object):
    """ """

    def __init__(self, context):
        self.context = context



# -*- coding: utf-8 -*-
from design.plone.contenttypes.controlpanels.settings import IDesignPloneSettings
from plone import api
from plone.restapi.behaviors import IBlocks
from uuid import uuid4
from zope.interface import implementer

import json
import logging
import six


HAVE_REST_API_PRE_961 = False

try:
    # plone 6.0.11 with last plone.restapi>9.6.0
    from plone.restapi.indexers import get_blocks_text
    from plone.restapi.indexers import text_strip

except ImportError:
    # plone 6.0.10.1 with plone.restapi<9.6.1
    HAVE_REST_API_PRE_961 = True
    from plone.restapi.indexers import SearchableText_blocks


logger = logging.getLogger(__name__)


def get_settings_for_language(field):
    values = api.portal.get_registry_record(
        field, interface=IDesignPloneSettings, default=[]
    )
    if not values:
        return []
    if not isinstance(values, six.text_type):
        return values
    try:
        json_data = json.loads(values)
    except Exception as e:
        logger.exception(e)
        return values
    lang = api.portal.get_current_language()
    return json_data.get(lang, [])


def create_default_blocks(context):
    title_uuid = str(uuid4())
    context.blocks = {title_uuid: {"@type": "title"}}
    context.blocks_layout = {"items": [title_uuid]}


def text_in_block(blocks):
    @implementer(IBlocks)
    class FakeObject(object):
        """
        We use a fake object to use SearchableText Indexer
        """

        def Subject(self):
            return ""

        def __init__(self, blocks, blocks_layout):
            self.blocks = blocks
            self.blocks_layout = blocks_layout
            self.id = ""
            self.title = ""
            self.description = ""

    if not blocks:
        return None

    fakeObj = FakeObject(blocks.get("blocks", ""), blocks.get("blocks_layout", ""))

    if HAVE_REST_API_PRE_961:
        return SearchableText_blocks(fakeObj)()
    else:
        blocks_text = get_blocks_text(fakeObj)
        return text_strip(blocks_text)



# -*- coding: utf-8 -*-
from Acquisition import aq_base
from collective.volto.blocksfield.field import BlocksField
from copy import deepcopy
from design.plone.contenttypes.controlpanels.settings import IDesignPloneSettings
from design.plone.contenttypes.setuphandlers import remove_blocks_behavior
from design.plone.contenttypes.upgrades.draftjs_converter import to_draftjs
from design.plone.contenttypes.utils import create_default_blocks
from plone import api
from plone.app.textfield.value import RichTextValue
from plone.app.upgrade.utils import installOrReinstallProduct
from plone.base.interfaces.syndication import ISiteSyndicationSettings
from plone.dexterity.utils import iterSchemata
from plone.registry.interfaces import IRegistry
from Products.CMFPlone.interfaces import ISelectableConstrainTypes
from redturtle.bandi.interfaces.settings import IBandoSettings
from transaction import commit
from uuid import uuid4
from z3c.relationfield import RelationValue
from zope.component import getUtility
from zope.event import notify
from zope.intid.interfaces import IIntIds
from zope.lifecycleevent import ObjectModifiedEvent
from zope.schema import getFields
from design.plone.contenttypes.events.common import SUBFOLDERS_MAPPING

import json
import logging
import six


logger = logging.getLogger(__name__)

DEFAULT_PROFILE = "profile-design.plone.contenttypes:default"

# standard upgrades #


def update_profile(context, profile, run_dependencies=True):
    context.runImportStepFromProfile(DEFAULT_PROFILE, profile, run_dependencies)


def update_actions(context):
    update_profile(context, "actions")


def update_types(context):
    update_profile(context, "typeinfo")


def update_rolemap(context):
    update_profile(context, "rolemap")


def update_registry(context):
    update_profile(context, "plone.app.registry", run_dependencies=False)


def update_catalog(context):
    update_profile(context, "catalog")


def update_controlpanel(context):
    update_profile(context, "controlpanel")


def reindex_catalog(context, idxs):
    pc = api.portal.get_tool(name="portal_catalog")
    brains = pc()
    for brain in brains:
        if idxs:
            brain.getObject().reindexObject(idxs=idxs)
        else:
            brain.getObject().reindexObject()


def remap_fields(mapping):
    pc = api.portal.get_tool(name="portal_catalog")
    brains = pc()
    tot = len(brains)
    logger.info("Trovati {} elementi da sistemare.".format(tot))
    # remap fields
    for brain in brains:
        item = brain.getObject()
        for old, new in mapping.items():
            value = getattr(item, old, None)
            if value:
                setattr(item, new, value)
                setattr(item, old, None)
                logger.info(
                    "- {url}: {old} -> {new}".format(
                        url=brain.getURL(), old=old, new=new
                    )
                )
                delattr(item, old)


# custom ones #


def to_1001(context):
    update_types(context)

    # cleanup event behaviors
    portal_types = api.portal.get_tool(name="portal_types")
    behaviors = portal_types["Event"].behaviors
    to_remove = [
        "design.plone.contenttypes.behavior.luoghi_correlati",
        "design.plone.contenttypes.behavior.argomenti_evento",
        "design.plone.contenttypes.behavior.additional_help_infos_evento",
    ]
    portal_types["Event"].behaviors = tuple(
        [x for x in behaviors if x not in to_remove]
    )

    mapping = {
        # "descrizione_destinatari": "a_chi_si_rivolge",
        "canale_fisico": "dove_rivolgersi_extra",
        "canale_fisico_prenotazione": "prenota_appuntamento",
        "fasi_scadenze": "tempi_e_scadenze",
        "sedi_e_luoghi": "dove_rivolgersi",
        "box_aiuto": "ulteriori_informazioni",
        "riferimento_telefonico_luogo": "telefono",
        "riferimento_mail_luogo": "email",
    }
    remap_fields(mapping=mapping)


def to_1003(context):
    update_types(context)

    mapping = {
        "unita_amministrativa_responsabile": "unita_amministrative_responsabili",  # noqa
        "sedi": "sede",
        "contatto_reperibilita": "reperibilita",
        "evento_supportato_da": "supportato_da",
    }
    remap_fields(mapping=mapping)


def to_1005(context):
    def fix_index(blocks):
        for block in blocks.values():
            if block.get("@type", "") == "listing":
                for query in block.get("query", []):
                    if (
                        query["i"] == "argomenti_correlati"
                        or query["i"] == "tassonomia_argomenti"
                    ):  # noqa
                        query["i"] = "argomenti"
                        logger.info(" - {}".format(brain.getURL()))

    # fix root
    portal = api.portal.get()
    portal_blocks = json.loads(portal.blocks)
    fix_index(portal_blocks)
    portal.blocks = json.dumps(portal_blocks)

    logger.info("Fixing listing blocks.")
    for brain in api.content.find(object_provides="plone.restapi.behaviors.IBlocks"):
        item = brain.getObject()
        blocks = deepcopy(getattr(item, "blocks", {}))
        if blocks:
            fix_index(blocks)
            item.blocks = blocks
    logger.info("** Reindexing items that refers to an argument **")
    for brain in api.portal.get_tool("portal_catalog")():
        item = brain.getObject()
        if getattr(item.aq_base, "tassonomia_argomenti", []):
            logger.info(" - {}".format(brain.getURL()))
            item.reindexObject(idxs=["argomenti"])


def to_1006(context):
    def fix_index(blocks):
        for block in blocks.values():
            if block.get("@type", "") == "listing":
                for query in block.get("query", []):
                    if (
                        query["i"] == "argomenti_correlati"
                        or query["i"] == "tassonomia_argomenti"
                        or query["i"] == "argomenti"
                    ):  # noqa
                        query["i"] = "argomenti"
                        query["v"] = [x.Title for x in api.content.find(UID=query["v"])]
                        logger.info(" - {}".format(brain.getURL()))

    # fix root
    portal = api.portal.get()
    portal_blocks = json.loads(portal.blocks)
    fix_index(portal_blocks)
    portal.blocks = json.dumps(portal_blocks)

    logger.info("Fixing listing blocks.")
    for brain in api.content.find(object_provides="plone.restapi.behaviors.IBlocks"):
        item = brain.getObject()
        blocks = deepcopy(getattr(item, "blocks", {}))
        if blocks:
            fix_index(blocks)
            item.blocks = blocks


def to_1007(context):
    for brain in api.content.find(portal_type="Persona"):
        item = brain.getObject()
        if item.email:
            item.email = [item.email]
        if item.telefono:
            item.telefono = [item.telefono]


def to_1008(context):
    installOrReinstallProduct(api.portal.get(), "redturtle.bandi")


def to_1009(context):
    def fix_index(blocks):
        """
        revert to tassonomia_argomenti
        """
        for block in blocks.values():
            if block.get("@type", "") == "listing":
                for query in block.get("query", []):
                    if query["i"] == "argomenti":
                        query["i"] = "tassonomia_argomenti"
                        logger.info(" - {}".format(brain.getURL()))

    # fix root
    portal = api.portal.get()
    portal_blocks = json.loads(portal.blocks)
    fix_index(portal_blocks)
    portal.blocks = json.dumps(portal_blocks)

    logger.info("Fixing listing blocks.")
    for brain in api.content.find(object_provides="plone.restapi.behaviors.IBlocks"):
        item = brain.getObject()
        blocks = deepcopy(getattr(item, "blocks", {}))
        if blocks:
            fix_index(blocks)
            item.blocks = blocks
    logger.info("** Reindexing items that refers to an argument **")
    for brain in api.portal.get_tool("portal_catalog")():
        item = brain.getObject()
        if getattr(item.aq_base, "tassonomia_argomenti", []):
            logger.info(" - {}".format(brain.getURL()))
            item.reindexObject(idxs=["tassonomia_argomenti"])


def to_1010(context):
    pc = api.portal.get_tool(name="portal_catalog")
    pc.manage_reindexIndex(ids=["event_location"])


def to_1013(context):
    def fix_template_name(blocks):
        """
        revert to tassonomia_argomenti
        """
        found = False
        for block in blocks.values():
            if (
                block.get("@type", "") == "listing"
                and block.get("template", "") == "imageGallery"
            ):
                block["template"] = "photogallery"
                found = True
        return found

    # fix root
    logger.info('Changing listing block template from "imageGallery" to "photogallery')
    portal = api.portal.get()
    portal_blocks = json.loads(portal.blocks)
    to_update = fix_template_name(portal_blocks)
    fixed_items = []
    if to_update:
        portal.blocks = json.dumps(portal_blocks)
        fixed_items.append("Root")
    i = 0
    brains = api.content.find(object_provides="plone.restapi.behaviors.IBlocks")
    tot = len(brains)
    for brain in brains:
        i += 1
        if i % 1000 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
        item = brain.getObject()
        blocks = deepcopy(getattr(item, "blocks", {}))
        if blocks:
            to_update = fix_template_name(blocks)
            if to_update:
                item.blocks = blocks
                fixed_items.append(brain.getPath())

    logger.info("Finish")
    if fixed_items:
        logger.info("Updated items:")
        for fixed in fixed_items:
            logger.info("- {}".format(fixed))
    else:
        logger.info("No items affected.")


def to_1014(context):
    update_types(context)
    portal_types = api.portal.get_tool(name="portal_types")
    portal_types["Bando"].behaviors = tuple(
        [
            x
            for x in portal_types["Bando"].behaviors
            if x != "design.plone.contenttypes.behavior.argomenti"
        ]
    )


def to_1015(context):
    update_types(context)

    # cleanup trasparenza behavior from CTs
    portal_types = api.portal.get_tool(name="portal_types")
    service_behaviors = portal_types["Servizio"].behaviors
    to_remove = [
        "design.plone.contenttypes.behavior.trasparenza",
    ]
    portal_types["Servizio"].behaviors = tuple(
        [x for x in service_behaviors if x not in to_remove]
    )
    persona_behaviors = portal_types["Persona"].behaviors
    portal_types["Persona"].behaviors = tuple(
        [x for x in persona_behaviors if x not in to_remove]
    )


def to_1016(context):
    section_ids = ["amministrazione", "servizi", "novita", "documenti-e-dati"]
    sections = []
    portal = api.portal.get()
    for id in section_ids:
        item = portal.get(id, None)
        if item:
            sections.append({"title": item.title, "linkUrl": [item.UID()]})
    settings = [{"rootPath": "/", "items": sections}]
    api.portal.set_registry_record(
        "search_sections",
        json.dumps(settings),
        interface=IDesignPloneSettings,
    )


def to_2000(context):  # noqa: C901
    # remove volto.blocks behavior from news and events and add new one
    update_types(context)
    portal_types = api.portal.get_tool(name="portal_types")
    for ptype in ["News Item", "Event"]:
        portal_types[ptype].behaviors = tuple(
            [x for x in portal_types[ptype].behaviors if x != "volto.blocks"]
        )
    portal_types["Pagina Argomento"].behaviors = tuple(
        [
            x
            for x in portal_types["Pagina Argomento"].behaviors
            if x != "design.plone.contenttypes.behavior.additional_help_infos"
        ]
    )
    # now copy values in new fields
    pc = api.portal.get_tool(name="portal_catalog")
    brains = pc()
    tot = len(brains)
    i = 0
    logger.info("### START CONVERSION FIELDS RICHTEXT -> DRAFTJS ###")
    for brain in brains:
        i += 1
        if i % 1000 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
        item = brain.getObject()
        if brain.portal_type in ["Event", "News Item"]:
            blocks = getattr(item, "blocks", {})
            blocks_layout = getattr(item, "blocks_layout", {"items": []})["items"]
            if not blocks:
                continue
            title_uid = ""
            new_blocks = {}
            for uid, block in blocks.items():
                if block.get("@type", "") == "title":
                    title_uid = uid
                else:
                    new_blocks[uid] = block
            item.descrizione_estesa = {
                "blocks": new_blocks,
                "blocks_layout": {
                    "items": [x for x in blocks_layout if x != title_uid]
                },
            }
            item.blocks = None
            item.blocks_layout = None
        for schema in iterSchemata(item):
            for name, field in getFields(schema).items():
                if not isinstance(field, BlocksField):
                    continue
                value = field.get(item)
                if not value:
                    continue
                if isinstance(value, six.string_types):
                    value = "<p>{}</p>".format(value)
                elif isinstance(value, RichTextValue):
                    value = value.raw
                else:
                    continue
                if value == "<p><br></p>":
                    value = ""
                try:
                    new_value = to_draftjs(value)
                except Exception as e:
                    logger.error(
                        "[NOT MIGRATED] - {}: {}".format(brain.getPath(), name)
                    )
                    raise e
                setattr(item, name, new_value)


def to_2002(context):
    """Per l'aggiornamento del vocabolario tipologie_persona, sistemiamo
    tutti quelli già presenti.
    """
    type_mapping = {
        "altro": "Altro tipo",
        "politica": "Politica",
        "amministrativa": "Amministrativa",
    }
    logger.info("Fixing 'Tipologia Persona'...")
    fixed_total = 0
    for brain in api.content.find(portal_type="Persona"):
        item = brain.getObject()
        if (
            hasattr(item, "tipologia_persona")
            and item.tipologia_persona in type_mapping
        ):  # noqa
            item.tipologia_persona = type_mapping[item.tipologia_persona]
            fixed_total += 1
        commit()
    logger.info("Fixing 'Tipologia Persona': DONE")
    logger.info("Updated {} objects".format(fixed_total))


def to_3000(context):
    """ """
    update_registry(context)
    update_controlpanel(context)
    multilanguage = [
        "tipologie_notizia",
        "tipologie_unita_organizzativa",
        # "tipologie_documento",
        "tipologie_persona",
    ]
    simple = ["lead_image_dimension", "search_sections"]
    old_entry = "design.plone.contenttypes.controlpanels.vocabularies.IVocabulariesControlPanel.{}"  # noqa
    for field in simple:
        value = api.portal.get_registry_record(old_entry.format(field))
        api.portal.set_registry_record(field, value, interface=IDesignPloneSettings)

    for field in multilanguage:
        try:
            value = api.portal.get_registry_record(old_entry.format(field))
            api.portal.set_registry_record(
                field,
                json.dumps({"it": value}),
                interface=IDesignPloneSettings,
            )
        except Exception:  # nosec
            # do not do anything
            continue

    context.runAllImportStepsFromProfile("profile-design.plone.contenttypes:to_3000")


def to_3101(context):
    intids = getUtility(IIntIds)
    logger.info("Fixing Documento references...")
    fixed_total = 0
    for brain in api.content.find(portal_type="Documento"):
        item = brain.getObject()
        for rel in getattr(item, "servizi_collegati", []):
            service = rel.to_object
            if service:
                service.altri_documenti.append(RelationValue(intids.getId(item)))
                notify(ObjectModifiedEvent(service))
                logger.info("Fixed item {}".format("/".join(service.getPhysicalPath())))

        if getattr(item, "servizi_collegati", []):
            delattr(item, "servizi_collegati")
            notify(ObjectModifiedEvent(item))
            fixed_total += 1
            logger.info("Fixed item {}".format("/".join(item.getPhysicalPath())))

    logger.info("Fixing 'Documento': DONE")
    logger.info("Updated {} objects Documento".format(fixed_total))


def to_3102(context):
    update_types(context)

    # cleanup trasparenza behavior from CTs
    portal_types = api.portal.get_tool(name="portal_types")
    to_remove = [
        "design.plone.contenttypes.behavior.trasparenza",
    ]
    for key, value in portal_types.items():
        ct_behaviors = getattr(value, "behaviors", None)
        if ct_behaviors is not None:
            portal_types[key].behaviors = tuple(
                [x for x in ct_behaviors if x not in to_remove]
            )


def to_volto13(context):  # noqa: C901
    # convert listing blocks with new standard

    logger.info("### START CONVERSION TO VOLTO 13: default => simpleCard ###")

    def fix_listing(blocks, url):
        for block in blocks.values():
            if block.get("@type", "") == "listing":
                if block.get("template", False) and not block.get("variation", False):
                    logger.error("- {}".format(url))
                if block.get("template", False) and block.get("variation", False):
                    logger.error("- {}".format(url))
                if block.get("variation", "") == "default":
                    block["variation"] = "simpleCard"
                    logger.info("- {}".format(url))

    # fix root
    portal = api.portal.get()
    portal_blocks = json.loads(portal.blocks)
    fix_listing(portal_blocks, portal.absolute_url())
    portal.blocks = json.dumps(portal_blocks)

    # fix blocks in contents
    pc = api.portal.get_tool(name="portal_catalog")
    brains = pc()
    tot = len(brains)
    i = 0
    for brain in brains:
        i += 1
        if i % 1000 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
        item = aq_base(brain.getObject())
        if getattr(item, "blocks", {}):
            blocks = deepcopy(item.blocks)
            if blocks:
                fix_listing(blocks, brain.getURL())
                item.blocks = blocks
        for schema in iterSchemata(item):
            # fix blocks in blocksfields
            for name, field in getFields(schema).items():
                if name == "blocks":
                    blocks = deepcopy(item.blocks)
                    if blocks:
                        fix_listing(blocks, brain.getURL())
                        item.blocks = blocks
                elif isinstance(field, BlocksField):
                    value = deepcopy(field.get(item))
                    if not value:
                        continue
                    if isinstance(value, str):
                        if value == "":
                            setattr(
                                item,
                                name,
                                {"blocks": {}, "blocks_layout": {"items": []}},
                            )
                            continue
                    try:
                        blocks = value.get("blocks", {})
                    except AttributeError:
                        logger.warning(
                            "[RICHTEXT] - {} (not converted)".format(brain.getURL())
                        )
                    if blocks:
                        fix_listing(blocks, brain.getURL())
                        setattr(item, name, value)


def to_3400(context):  # noqa: C901
    logger.info("### START CONVERSION BLOCKS: newsHome -> highlitedContent ###")

    def fix_block(blocks, url):
        for block in blocks.values():
            if block.get("@type", "") == "newsHome":
                block["@type"] = "highlitedContent"

    # fix root
    portal = api.portal.get()
    portal_blocks = json.loads(portal.blocks)
    fix_block(portal_blocks, portal.absolute_url())
    portal.blocks = json.dumps(portal_blocks)

    # fix blocks in contents
    pc = api.portal.get_tool(name="portal_catalog")
    brains = pc()
    tot = len(brains)
    i = 0
    for brain in brains:
        i += 1
        if i % 1000 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
        item = aq_base(brain.getObject())
        if getattr(item, "blocks", {}):
            blocks = deepcopy(item.blocks)
            if blocks:
                fix_block(blocks, brain.getURL())
                item.blocks = blocks
        for schema in iterSchemata(item):
            # fix blocks in blocksfields
            for name, field in getFields(schema).items():
                if name == "blocks":
                    blocks = deepcopy(item.blocks)
                    if blocks:
                        fix_block(blocks, brain.getURL())
                        item.blocks = blocks
                elif isinstance(field, BlocksField):
                    value = deepcopy(field.get(item))
                    if not value:
                        continue
                    if isinstance(value, str):
                        if value == "":
                            setattr(
                                item,
                                name,
                                {"blocks": {}, "blocks_layout": {"items": []}},
                            )
                            continue
                    try:
                        blocks = value.get("blocks", {})
                    except AttributeError:
                        logger.warning(
                            "[RICHTEXT] - {} (not converted)".format(brain.getURL())
                        )
                    if blocks:
                        fix_block(blocks, brain.getURL())
                        setattr(item, name, value)


def to_3401(context):  # noqa: C901
    logger.info("File type can now be added inside a CartellaModulistica")
    update_types(context)


def to_3500(context):
    logger.info("Add new index and reindex UO")
    update_catalog(context)

    # remove unused index
    catalog = api.portal.get_tool(name="portal_catalog")
    if "office_venue" in catalog.indexes():
        catalog.manage_delIndex("office_venue")

    # reindex
    brains = api.content.find(portal_type="UnitaOrganizzativa")
    tot = len(brains)
    logger.info("Found {} UO.".format(tot))
    i = 0
    for brain in brains:
        i += 1
        if i % 1000 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
        uo = brain.getObject()
        uo.reindexObject(idxs=["uo_location", "tipologia_organizzazione"])


def to_3501(context):
    logger.info("Reindex UO for new SearchableText fields")

    brains = api.content.find(portal_type="UnitaOrganizzativa")
    tot = len(brains)
    logger.info("Found {} UO.".format(tot))
    i = 0
    for brain in brains:
        i += 1
        if i % 1000 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
        uo = brain.getObject()
        uo.reindexObject(idxs=["SearchableText"])


def to_3600(context):
    logger.info("Enable kitconcept.seo behavior")
    types_list = [
        "UnitaOrganizzativa",
        "Bando",
        "Subsite",
        "Venue",
        "Persona",
        "Event",
        "News Item",
        "Document",
        "Documento",
        "Servizio",
        "CartellaModulistica",
        "Pagina Argomento",
    ]
    portal_types = api.portal.get_tool(name="portal_types")
    for ct_type in types_list:
        if "kitconcept.seo" not in portal_types[ct_type].behaviors:
            portal_types[ct_type].behaviors += ("kitconcept.seo",)
            logger.info("Enabled kitconcept.seo on: {}".format(ct_type))

    logger.info("Bandi customizations")
    update_catalog(context)
    api.portal.set_registry_record("default_ente", (), interface=IBandoSettings)

    portal_types = api.portal.get_tool(name="portal_types")
    portal_types["Bando Folder Deepening"].allowed_content_types = (
        "Modulo",
        "File",
        "Link",
    )
    portal_types["Bando"].default_view = "view"
    portal_types["Bando"].view_methods = ("view",)

    logger.info("Reindex SearchableText")
    pc = api.portal.get_tool(name="portal_catalog")
    pc.reindexIndex("SearchableText", context.REQUEST)

    logger.info("Reindex Bandi")
    i = 0
    brains = api.content.find(portal_type="Bando")
    tot = len(brains)
    for brain in brains:
        i += 1
        if i % 1000 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
        bando = brain.getObject()
        bando.reindexObject(idxs=["ufficio_responsabile_bando", "Subject_bando"])


def to_3700(context):
    logger.info("Set show_modified_default as True")

    api.portal.set_registry_record(
        "show_modified_default", True, interface=IDesignPloneSettings
    )


def to_3800(context):
    logger.info("Fix Venue addable types")

    portal_types = api.portal.get_tool(name="portal_types")
    portal_types["Venue"].allowed_content_types = (
        "Folder",
        "Image",
        "File",
        "Link",
    )


def to_3900(context):
    logger.info("Add new metadata: ruolo")

    brains = api.content.find(portal_type="Persona")
    for brain in brains:
        persona = brain.getObject()
        persona.reindexObject()


def to_4000(context):
    logger.info("Move ruolo to a choice field")
    ruoli = {"it": [], "en": []}
    brains = api.content.find(portal_type="Persona")
    for brain in brains:
        persona = brain.getObject()
        ruolo = getattr(persona, "ruolo", "")
        lang = brain.language
        if ruolo not in ruoli[lang]:
            ruoli[lang].append(ruolo)

    if api.portal.get_registry_record(
        "ruoli_persona", interface=IDesignPloneSettings, default=None
    ):
        api.portal.set_registry_record(
            "ruoli_persona", json.dumps(ruoli), interface=IDesignPloneSettings
        )


def to_4100(context):
    logger.info("Add constrainttypes behavior to Document")

    portal_types = api.portal.get_tool(name="portal_types")
    document_behaviors = list(portal_types["Document"].behaviors) + [
        "plone.constraintypes"
    ]
    portal_types["Document"].behaviors = tuple(document_behaviors)


def to_4200(context):
    logger.info("Add criteria and indexes to Persona")

    update_catalog(context)
    update_registry(context)

    brains = api.content.find(portal_type="Persona")
    for brain in brains:
        persona = brain.getObject()
        persona.reindexObject(idxs=["data_conclusione_incarico"])


def to_5000(context):
    logger.info("Enable preview_image behavior in all content types")

    portal_types = api.portal.get_tool(name="portal_types")

    for portal_type, fti in portal_types.items():
        behaviors = list(getattr(fti, "behaviors", ()))
        if not behaviors:
            continue
        if portal_type == "Document":
            behaviors = [
                x
                for x in behaviors
                if x not in ["plone.leadimage", "volto.preview_image"]
            ]
            behaviors.extend(["plone.leadimage", "volto.preview_image"])
        else:
            if "plone.leadimage" not in behaviors or "volto.preview_image" in behaviors:
                continue
        behaviors.insert(behaviors.index("plone.leadimage") + 1, "volto.preview_image")
        fti.behaviors = tuple(behaviors)

    logger.info("Move immagine_testata to image")
    catalog = api.portal.get_tool("portal_catalog")
    i = 0
    brains = catalog()
    tot = len(brains)
    for brain in brains:
        i += 1
        if i % 500 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
        obj = brain.getObject()
        if "image" in obj.keys():
            api.content.rename(obj=obj["image"], new_id="image-1")
        if brain.portal_type == "Document":
            immagine_testata = getattr(obj, "immagine_testata", None)
            if immagine_testata:
                obj.image = immagine_testata
                obj.immagine_testata = None
        catalog.catalog_object(obj)


def to_5001(context):
    catalog = api.portal.get_tool("portal_catalog")
    i = 0
    brains = catalog(portal_type="Pagina Argomento")
    tot = len(brains)
    logger.info("Add icona metadata to {}".format(tot))
    for brain in brains:
        i += 1
        if i % 500 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
        obj = brain.getObject()
        catalog.catalog_object(obj)


def to_5002(context):
    """
    Reindex non-folderish items because there were some metadata not updated
    """
    catalog = api.portal.get_tool("portal_catalog")
    i = 0
    brains = catalog(is_folderish=False)
    tot = len(brains)
    for brain in brains:
        i += 1
        if i % 500 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
        obj = brain.getObject()
        catalog.catalog_object(obj)


def to_5200(context):
    """
    add new behavior for Bando
    """
    portal_types = api.portal.get_tool(name="portal_types")
    fti = portal_types["Bando"]
    behaviors = list(getattr(fti, "behaviors", ()))
    if "design.plone.contenttypes.behavior.update_note" not in behaviors:
        behaviors.append("design.plone.contenttypes.behavior.update_note")
        fti.behaviors = tuple(behaviors)


def to_5210(context):
    logger.info("Enable preview_image behavior in Bandi content types")

    portal_types = api.portal.get_tool(name="portal_types")
    fti = portal_types["Bando"]
    behaviors = list(getattr(fti, "behaviors", ()))
    if "volto.preview_image" not in behaviors:
        behaviors.append("volto.preview_image")
        fti.behaviors = tuple(behaviors)


def to_5220(context):
    """
    Reindex Venues
    """
    logger.info("Reindex SearchableText for Venue items.")
    catalog = api.portal.get_tool("portal_catalog")
    i = 0
    brains = catalog(portal_type="Venue")
    tot = len(brains)
    for brain in brains:
        i += 1
        if i % 500 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
        obj = brain.getObject()
        obj.reindexObject(idxs=["SearchableText", "object_provides"])


def to_5300(context):
    update_profile(context, "plone-difftool")
    update_profile(context, "repositorytool")

    portal_types = api.portal.get_tool(name="portal_types")
    for portal_type, fti in portal_types.items():
        if portal_type in [
            "CartellaModulistica",
            "Documento",
            "Link",
            "Pagina Argomento",
            "Persona",
            "Servizio",
            "UnitaOrganizzativa",
            "Venue",
        ]:
            behaviors = list(getattr(fti, "behaviors", ()))
            if "plone.versioning" not in behaviors:
                behaviors.append("plone.versioning")
                fti.behaviors = tuple(behaviors)


def to_5310(context):
    """
    Reindex Bandi
    """
    logger.info("Reindex SearchableText for Bandi items.")
    catalog = api.portal.get_tool("portal_catalog")
    i = 0
    brains = catalog(portal_type="Bando")
    tot = len(brains)
    for brain in brains:
        i += 1
        if i % 500 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
        brain.getObject().reindexObject(idxs=["SearchableText"])


def to_5400(context):
    logger.info('Remove "volto.blocks" behavior from News Item and Event.')
    remove_blocks_behavior(context)


def to_5410(context):
    # cleanup Document behaviors
    portal_types = api.portal.get_tool(name="portal_types")
    behaviors = portal_types["Document"].behaviors
    to_remove = [
        "plone.tableofcontents",
    ]
    portal_types["Document"].behaviors = tuple(
        [x for x in behaviors if x not in to_remove]
    )


def to_5500(context):
    update_registry(context)
    update_catalog(context)

    argomenti_mapping = {
        x.Title: x.UID for x in api.content.find(portal_type="Pagina Argomento")
    }

    def fix_block(blocks, argomenti_mapping):
        for block in blocks.values():
            if block.get("@type", "") == "listing":
                for query in block.get("querystring", {}).get("query", []):
                    if query["i"] == "tassonomia_argomenti":
                        new_values = []
                        for v in query["v"]:
                            uid = argomenti_mapping.get(v, "")
                            if uid:
                                new_values.append(uid)
                        query["i"] = "tassonomia_argomenti_uid"
                        query["v"] = new_values
                        logger.info(" - {}".format(brain.getURL()))

    pc = api.portal.get_tool(name="portal_catalog")
    brains = pc()
    tot = len(brains)
    i = 0
    for brain in brains:
        i += 1
        if i % 1000 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
        item_obj = brain.getObject()
        item = aq_base(brain.getObject())

        # reindex argomenti indexes
        if brain.tassonomia_argomenti:
            item_obj.reindexObject(
                idxs=["tassonomia_argomenti", "tassonomia_argomenti_uid"]
            )

        if getattr(item, "blocks", {}):
            blocks = deepcopy(item.blocks)
            if blocks:
                fix_block(blocks, argomenti_mapping)
                item.blocks = blocks
        for schema in iterSchemata(item):
            # fix blocks in blocksfields
            for name, field in getFields(schema).items():
                if name == "blocks":
                    blocks = deepcopy(item.blocks)
                    if blocks:
                        fix_block(blocks, argomenti_mapping)
                        item.blocks = blocks
                elif isinstance(field, BlocksField):
                    value = deepcopy(field.get(item))
                    if not value:
                        continue
                    blocks = value.get("blocks", {})
                    if blocks:
                        fix_block(blocks, argomenti_mapping)
                        setattr(item, name, value)


def to_6000(context):
    """ """
    logger.info(
        "Convert behavior: collective.dexteritytextindexer => plone.textindexer"  # noqa
    )
    portal_types = api.portal.get_tool(name="portal_types")
    for fti in portal_types.values():
        behaviors = []
        for behavior in getattr(fti, "behaviors", ()):
            if behavior == "collective.dexteritytextindexer":
                behavior = "plone.textindexer"
            behaviors.append(behavior)

        fti.behaviors = tuple(behaviors)


def to_6010(context):
    """ """
    update_types(context)
    update_registry(context)
    update_catalog(context)
    update_rolemap(context)


def to_6011(context):
    """ """
    update_types(context)


def migrate_pdc_and_incarico(context):
    # Cannot test rn, blind coding
    update_types(context)
    update_registry(context)
    update_catalog(context)
    update_rolemap(context)
    # "field name in original ct": "field name in new ct"
    type_mapping = {
        "Persona": {
            "PDC": {
                "telefono": "telefono",
                "fax": "fax",
                "email": "email",
                "pec": "pec",
            },  # noqa
            "Incarico": {
                # HOW? Need taxonomies also
                # We could do:
                # persona.ruolo.title = incarico.title
                # persona.items.compensi = incarico.items.compensi?
                "ruolo?": "incarico?",
                # BlobFile to relation with Documento
                "atto_nomina": "atto_nomina",
                "data_conclusione_incarico": "data_conclusione_incarico",
                "data_insediamento": "data_insediamento",
            },
        },
        "UnitaOrganizzativa": {
            "PDC": {
                "telefono": "telefono",
                "fax": "fax",
                "email": "email",
                "pec": "pec",
                "web": "web",
            },
        },
        # TODO: tbc
        "Event": {
            "PDC": {
                "telefono": "telefono",
                "fax": "fax",
                "email": "email",
                "pec": "pec",
            },  # noqa
        },
        # TODO: tbc
        "Venue": {
            "PDC": {
                "riferimento_telefonico_struttura": "telefono",
                "riferimento_fax_struttura": "fax",
                "riferimento_mail_struttura": "email",
                "riferimento_pec_struttura": "pec",
            },
        },
        # TODO: tbc
        "Servizio": {
            # questi non sono presenti sul ct originale
            "PDC": {
                "telefono": "telefono",
                "fax": "fax",
                "email": "email",
                "pec": "pec",
            },  # noqa
        },
    }

    def createIncaricoAndMigratePersona(portal_type):
        # Taxonomies work needs to be completed before, blind coding ahead
        if portal_type == "Persona":
            fixed_total = 0
            for brain in api.content.find(portal_type=portal_type):
                item = brain.getObject()
                atto_nomina = item.atto_nomina
                logger.info(f"Fixing Punto di Contatto for '{item.title}'...")  # noqa
                file_bog = api.content.find(context=item, depth=1, id="atti-nomina")
                if not file_bog:
                    try:
                        file_bog = api.content.create(
                            type="Document",
                            id="atti-nomina",
                            title="Atti Nomina",
                            container=item,
                        )
                    except Exception:
                        logger.error("Error", Exception)

                try:
                    new_atto_nomina = api.content.create(
                        type="File",
                        id=atto_nomina.id,
                        title=atto_nomina.title,
                        container=item,
                        **{"file": atto_nomina},
                    )
                    intids = getUtility(IIntIds)
                    relation = [RelationValue(intids.getId(new_atto_nomina))]
                    incarico = api.content.create(
                        type="Incarico", title=item.ruolo.title, container=item
                    )
                    incarico.atto_nomina = relation
                    item.atto_nomina = None
                    fixed_total += 1
                    logger.info(
                        f"Fixing Punto di Contatto for '{item.title}'...:DONE"
                    )  # noqa
                except Exception:
                    logger.error("Error", Exception)
            logger.info("Updated {} objects".format(fixed_total))

        pass

    def createPDCandMigrateOldCTs(portal_type):
        logger.info(f"Fixing Punto di Contatto for '{portal_type}'...")  # noqa
        fixed_total = 0
        mapping = None
        # mapping = type_mapping[portal_type]["PDC"]
        # Reenable mapping to use
        if not mapping:
            logger.info(f"No need to fix Punto di Contatto for '{portal_type}: DONE")
            return
        for brain in api.content.find(portal_type=portal_type):
            item = brain.getObject()
            kwargs = {"value_punto_contatto": [], "persona": []}
            for key, value in mapping.items():
                if hasattr(item, key):
                    kwargs["value_punto_contatto"].append(
                        {"pdc_type": value, "pdc_value": item[key]}
                    )

            new_pdc = api.content.create(
                type="PuntoDiContatto",
                title=f"Punto di Contatto {item.id}",
                container=item,
                **kwargs,
            )
            intids = getUtility(IIntIds)
            item.contact_info = [RelationValue(intids.getId(new_pdc))]
            fixed_total += 1
            commit()

        logger.info(f"Fixing Punto di Contatto for '{portal_type}: DONE")
        logger.info("Updated {} objects".format(fixed_total))

    for pt in type_mapping:
        logger.info(
            "Migrating existing CTs for use with new Incarico and PDC Content Types"
        )
        createPDCandMigrateOldCTs(pt)
        createIncaricoAndMigratePersona(pt)


class colors(object):
    GREEN = "\033[92m"
    ENDC = "\033[0m"
    RED = "\033[91m"
    DARKCYAN = "\033[36m"
    YELLOW = "\033[93m"


def update_uo_contact_info(context):
    brains = api.portal.get_tool("portal_catalog")(portal_type="UnitaOrganizzativa")
    logger.info(
        f"{colors.DARKCYAN} Inizio la pulzia delle {len(brains)} UO campo contact_info {colors.ENDC}"  # noqa
    )
    for brain in brains:
        obj = brain.getObject()
        if type(obj.contact_info) == dict:  # noqa
            del obj.contact_info
            logger.info(
                f"{colors.GREEN} Modifica della UO senza punto di contatto {colors.ENDC}"  # noqa
            )


def readd_tassonomia_argomenti_uid(context):
    logger.info(
        f"{colors.DARKCYAN} Aggiungo la tassonomia_argomenti_uid e reindicizzo{colors.ENDC}"  # noqa
    )
    update_catalog(context)
    update_registry(context)
    idxs = ["tassonomia_argomenti_uid", "tassonomia_argomenti"]
    reindex_catalog(context, idxs)


def update_ruolo_indexing(context):
    logger.info(
        f"{colors.DARKCYAN} Reindex del ruolo nelle persone {colors.ENDC}"  # noqa
    )
    idxs = ["ruolo"]
    pc = api.portal.get_tool("portal_catalog")
    brains = pc(portal_type="Persona")
    for brain in brains:
        persona = brain.getObject()
        persona.reindexObject(idxs=idxs)


def fix_ctaxonomy_indexes_and_metadata(context):
    logger.info(f"{colors.DARKCYAN} Fix taxonomy indexes {colors.ENDC}")  # noqa
    bad_names = [
        "taxonomy_person_life_events",
        "taxonomy_business_events",
        "taxonomy_temi_dataset",
        "taxonomy_tipologia_documenti_albopretorio",
        "taxonomy_tipologia_documento",
        "taxonomy_tipologia_evento",
        "taxonomy_tipologia_frequenza_aggiornamento",
        "taxonomy_tipologia_incarico",
        "taxonomy_tipologia_licenze",
        "taxonomy_tipologia_luogo",
        "taxonomy_tipologia_notizia",
        "taxonomy_tipologia_organizzazione",
        "taxonomy_tipologia_pdc",
        "taxonomy_tipologia_stati_pratica",
    ]

    good_names = [name.replace("taxonomy_", "") for name in bad_names]
    catalog = api.portal.get_tool(name="portal_catalog")
    catalog_metadata = catalog.schema()
    catalog_indexes = catalog.indexes()

    for name in bad_names:
        # metadata
        if name in catalog_metadata:
            catalog.delColumn(name)
            logger.info(f"{colors.GREEN} Remove {name} from metadata {colors.ENDC}")

        # indexes
        if name in catalog_indexes:
            catalog.delIndex(name)
            logger.info(f"{colors.GREEN} Remove {name} from indexes {colors.ENDC}")

    context.runImportStepFromProfile(
        "design.plone.contenttypes:taxonomy", "collective.taxonomy"
    )
    brains = catalog(
        portal_type=[
            "News Item",
            "Event",
            "Venue",
            "Servizio",
            "Documento",
            "Dataset",
            "UnitaOrganizzativa",
            "Incarico",
            "Pratica",
        ]
    )
    logger.info(f"{colors.GREEN} Reindex contents with taxonomies {colors.ENDC}")
    for brain in brains:
        obj = brain.getObject()
        obj.reindexObject(idxs=good_names)
    logger.info(f"{colors.GREEN} End of update {colors.ENDC}")


def update_patrocinato_da(self):
    EMPTY_BLOCKS_FIELD = {"blocks": {}, "blocks_layout": {"items": []}}
    logger.info(
        f"{colors.DARKCYAN} Change patrocinato_da field in events {colors.ENDC}"
    )
    pc = api.portal.get_tool(name="portal_catalog")
    for brain in pc(portal_type="Event"):
        obj = brain.getObject()
        patrocinato_da = getattr(obj, "patrocinato_da")
        if patrocinato_da == EMPTY_BLOCKS_FIELD:
            logger.info(
                f"{colors.YELLOW} Nessuna informazione da modificare{colors.ENDC}"
            )
            continue
        url = obj.absolute_url()
        logger.info(f"{colors.GREEN} patrocinato_da ({url}){colors.ENDC}")

        setattr(
            obj,
            "patrocinato_da",
            {
                "blocks": {
                    "d252fe92-ce88-4866-b77d-501e7275cfc0": {
                        "@type": "text",
                        "text": {
                            "blocks": [
                                {
                                    "data": {},
                                    "depth": 0,
                                    "entityRanges": [],
                                    "inlineStyleRanges": [],
                                    "key": "e23it",
                                    "text": patrocinato_da,
                                    "type": "unstyled",
                                }
                            ],
                            "entityMap": {},
                        },
                    }
                },
                "blocks_layout": {"items": ["d252fe92-ce88-4866-b77d-501e7275cfc0"]},
            },
        )
        obj.reindexObject()
    logger.info(f"{colors.DARKCYAN} End of update {colors.ENDC}")


def update_folder_for_gallery(self):
    logger.info(f"{colors.DARKCYAN} Update events {colors.ENDC}")
    pc = api.portal.get_tool(name="portal_catalog")
    for brain in pc(portal_type="Event"):
        evento = brain.getObject()

        logger.info(f"{colors.DARKCYAN} Event: {evento.absolute_url()} {colors.ENDC}")
        if "multimedia" in evento.keys():
            renamed_event = api.content.rename(evento["multimedia"], new_id="immagini")
            renamed_event.title = "Immagini"
            renamed_event.reindexObject(idxs=["id", "title"])
            logger.info(f"{colors.GREEN} Rename multimedia {colors.ENDC}")

        if "video" not in evento.keys():
            galleria_video = api.content.create(
                container=evento,
                type="Document",
                title="Video",
                id="video",
            )
            create_default_blocks(context=galleria_video)

            # select  constraints
            constraintsGalleriaVideo = ISelectableConstrainTypes(galleria_video)
            constraintsGalleriaVideo.setConstrainTypesMode(1)
            constraintsGalleriaVideo.setLocallyAllowedTypes(("Link",))

            with api.env.adopt_roles(["Reviewer"]):
                api.content.transition(obj=galleria_video, transition="publish")

            logger.info(f"{colors.GREEN} Create video {colors.ENDC}")


def to_7009(context):
    portal_types = api.portal.get_tool(name="portal_types")
    behaviors = list(portal_types["Venue"].behaviors)
    if "plone.excludefromnavigation" in behaviors:
        return
    logger.info("Enable plone.excludefromnavigation behavior")
    behaviors.append("plone.excludefromnavigation")
    portal_types["Venue"].behaviors = tuple(behaviors)
    logger.info("Reindex Venue objects")
    brains = api.content.find(portal_type="Venue")
    tot = len(brains)
    i = 0
    for brain in brains:
        i += 1
        if i % 100 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
        venue = brain.getObject()
        if not getattr(venue, "exclude_from_nav", None):
            setattr(venue, "exclude_from_nav", False)
            venue.reindexObject(idxs=["exclude_from_nav"])


def to_7010(context):
    registry = getUtility(IRegistry)
    prefix = "Products.CMFPlone.interfaces.syndication.ISiteSyndicationSettings"

    # get the old values
    old_attributes = [
        attribute for attribute in registry.records if attribute.startswith(prefix)
    ]
    if not old_attributes:
        logger.info(
            f"{colors.GREEN} We already have the correct interface. Nothing to do here! {colors.ENDC}"  # noqa
        )
        return
    old_values = {
        attribute.split(".")[-1]: registry.records[attribute].value
        for attribute in old_attributes
    }
    logger.info(
        f"{colors.DARKCYAN} Deleting old ISiteSyndicationSettings Records {colors.ENDC}"
    )
    # delete the old records
    for attribute in old_attributes:
        del registry.records[attribute]

    # import the new interface
    logger.info(f"{colors.DARKCYAN} Setup new interface in the registry {colors.ENDC}")
    context.runImportStepFromProfile(
        "profile-design.plone.contenttypes:fix_syndication",
        "plone.app.registry",
        False,
    )
    logger.info(
        f"{colors.DARKCYAN} Set the old values into the new registry records{colors.ENDC}"  # noqa
    )
    # set the old values into the new records
    default_values = {
        "allowed": True,
        "allowed_feed_types": (
            "RSS|RSS 1.0",
            "rss.xml|RSS 2.0",
            "atom.xml|Atom",
            "itunes.xml|iTunes",
        ),
        "default_enabled": False,
        "max_items": 15,
        "render_body": False,
        "search_rss_enabled": True,
        "show_author_info": True,
        "show_syndication_button": False,
        "show_syndication_link": False,
        "site_rss_items": tuple(),
    }

    for attribute in old_values:
        # we could have None value and we can't set None with set_registry_record
        # so we can set the value to the default.
        if old_values[attribute] == None:  # noqa
            old_values[attribute] = default_values[attribute]
            logger.info(
                f"{colors.RED } Fix {attribute} to default: {old_values[attribute]} {colors.ENDC}"  # noqa
            )

        if attribute == "site_rss_items" and old_values[attribute]:
            logger.info(
                f"{colors.RED} Please manually fix {attribute} with old value"
                f" {old_values[attribute]} {colors.ENDC}"
            )
            continue

        logger.info(
            f"{colors.DARKCYAN} Set {attribute} to  {old_values[attribute]} {colors.ENDC}"  # noqa
        )
        api.portal.set_registry_record(
            name=attribute,
            value=old_values[attribute],
            interface=ISiteSyndicationSettings,
        )
    logger.info(
        f"{colors.GREEN}ISiteSyndicationSettings interface fixed! {colors.ENDC}"
    )


def to_7011(context):
    logger.info("Reindex Event and News to fix SearchableText indexing issue")

    brains = api.content.find(portal_type=["Event", "News Item"])
    tot = len(brains)
    logger.info("Found {} documents.".format(tot))
    i = 0
    for brain in brains:
        i += 1
        if i % 100 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
        doc = brain.getObject()
        doc.reindexObject(idxs=["SearchableText"])
    logger.info("Ends of reindex")


def to_7012(context):
    def has_empty_prezzo(value):
        if not value:
            return True
        if value == {"blocks": {}, "blocks_layout": {"items": []}}:
            return True
        blocks_layout = value.get("blocks_layout", {}).get("items", [])
        blocks = list(value.get("blocks", {}).values())
        if len(blocks_layout) == 1 and blocks[0] == {"@type": "text"}:
            return True
        return False

    logger.info("Set default value in prezzo field because now is required.")

    brains = api.content.find(portal_type=["Event"])
    tot = len(brains)
    logger.info(f"Found {tot} Events.")
    i = 0
    fixed = []
    for brain in brains:
        i += 1
        if i % 100 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
        event = brain.getObject()
        prezzo = getattr(event, "prezzo", None)
        if has_empty_prezzo(value=prezzo):
            fixed.append(brain.getPath())
            uid = str(uuid4())
            event.prezzo = {
                "blocks": {
                    uid: {
                        "@type": "text",
                        "text": {
                            "blocks": [
                                {
                                    "key": "fvsj1",
                                    "text": "Eventuali costi sono indicati nella descrizione dell’evento.",
                                    "type": "unstyled",
                                    "depth": 0,
                                    "inlineStyleRanges": [],
                                    "entityRanges": [],
                                    "data": {},
                                }
                            ],
                            "entityMap": {},
                        },
                    }
                },
                "blocks_layout": {"items": [uid]},
            }
    logger.info(f"Fixed {len(fixed)} Events.")


def update_pdc_with_pdc_desc(context):
    brains = api.content.find(portal_type="PuntoDiContatto")
    logger.info(f"Found {len(brains)} PuntoDiContatto content type")
    for brain in brains:
        pdc = brain.getObject()
        value_punto_contatto = getattr(pdc, "value_punto_contatto", [])
        if value_punto_contatto:
            for v in value_punto_contatto:
                if not v.get("pdc_desc", None):
                    v["pdc_desc"] = None
                    logger.info(f"Set pdc_desc for {pdc.absolute_url()}")

            pdc.value_punto_contatto = value_punto_contatto
    logger.info("Ends of update")


def add_canale_digitale_link_index(context):
    update_catalog(context)
    update_registry(context)
    brains = api.content.find(portal_type="Servizio")
    logger.info(f"Found {len(brains)} Servizio content type to reindex")
    for brain in brains:
        service = brain.getObject()
        service.reindexObject(idxs=["canale_digitale_link"])
        logger.info(f"Reindexed {service.absolute_url()}")
    logger.info("End of update, added index canale_digitale_link")


def to_7031(context):
    portal_types = api.portal.get_tool(name="portal_types")
    for ptype in ["News Item"]:
        portal_types[ptype].default_view = "view"
        portal_types[ptype].view_methods = ["view"]


def to_7100(context):
    installOrReinstallProduct(api.portal.get(), "collective.volto.enhancedlinks")
    # add behavior to modulo
    portal_types = api.portal.get_tool(name="portal_types")
    modulo_behaviors = [x for x in portal_types["Modulo"].behaviors]
    if "volto.enhanced_links_enabled" not in modulo_behaviors:
        modulo_behaviors.append("volto.enhanced_links_enabled")
    portal_types["Modulo"].behaviors = tuple(modulo_behaviors)

    # update index/metadata
    brains = api.content.find(portal_type=["File", "Image", "Modulo"])
    tot = len(brains)
    i = 0
    for brain in brains:
        i += 1
        if i % 100 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
        brain.getObject().reindexObject(idxs=["enhanced_links_enabled"])


def to_7200(context):
    update_catalog(context)
    # add behavior to Document and Folder
    bhv = "design.plone.contenttypes.behavior.exclude_from_search"
    portal_types = api.portal.get_tool(name="portal_types")
    for ptype in ["Document", "Folder"]:
        behaviors = [x for x in portal_types[ptype].behaviors]
        if bhv not in behaviors:
            behaviors.append(bhv)
        portal_types[ptype].behaviors = tuple(behaviors)

    # set True to all of already created children
    # update index/metadata
    brains = api.content.find(portal_type=[x for x in SUBFOLDERS_MAPPING.keys()])
    tot = len(brains)
    i = 0
    for brain in brains:
        i += 1
        if i % 100 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
        container = brain.getObject()
        mappings = SUBFOLDERS_MAPPING.get(container.portal_type, [])
        persona_old_mapping = [
            {
                "id": "foto-e-attivita-politica",
            },
            {"id": "curriculum-vitae"},
            {"id": "compensi"},
            {
                "id": "importi-di-viaggio-e-o-servizi",
            },
            {
                "id": "situazione-patrimoniale",
            },
            {
                "id": "dichiarazione-dei-redditi",
            },
            {
                "id": "spese-elettorali",
            },
            {
                "id": "variazione-situazione-patrimoniale",
            },
            {
                "id": "altre-cariche",
            },
        ]
        if container.portal_type == "Persona":
            # cleanup also some old-style (v2) folders
            mappings.extend(persona_old_mapping)

        for mapping in mappings:
            child = container.get(mapping["id"], None)
            if not child:
                continue
            if child.portal_type not in ["Folder", "Document"]:
                continue
            child.exclude_from_search = True

    catalog = api.portal.get_tool(name="portal_catalog")
    catalog.manage_reindexIndex(ids=["exclude_from_search"])



# -*- coding: utf-8 -*-
from design.plone.contenttypes.interfaces.punto_di_contatto import IPuntoDiContatto
from plone.dexterity.content import Container
from zope.interface import implementer


@implementer(IPuntoDiContatto)
class PuntoDiContatto(Container):
    """ """



# -*- coding: utf-8 -*-
from design.plone.contenttypes import _
from design.plone.contenttypes.interfaces import IDesignPloneContentType
from plone.supermodel import model
from zope import schema


class ICartellaModulistica(model.Schema, IDesignPloneContentType):
    """Cartella Modulistica"""

    visualize_files = schema.Bool(
        title=_("visualize_files_title", default="Mostra i PDF in anteprima"),
        description=_(
            "visulize_files_description",
            default="Permette di aprire l'anteprima di tutti i PDF di questa cartella"
            " in una tab separata, altrimenti i PDF vengono scaricati",
        ),
        required=False,
        default=False,
    )



# -*- coding: utf-8 -*-
from .interfaces import Correlati
from .interfaces import ICorrelati
from zope.interface import implementer


@implementer(ICorrelati)
class GetCorrelatiServizi(Correlati):
    """Adapter for upload file"""



from DateTime import DateTime
from openpyxl import Workbook
from openpyxl.styles import Alignment
from openpyxl.styles import Font
from openpyxl.styles import PatternFill
from openpyxl.utils import get_column_letter
from plone import api
from Products.Five import BrowserView

import io


FLAG = '<i class="fa-solid fa-check"></i>'


class CheckLuoghi(BrowserView):
    cds = None

    def is_anonymous(self):
        return api.user.is_anonymous()

    def information_dict(self, luogo):
        indirizzo = False
        if getattr(luogo, "city", "") and luogo.city.strip():
            if getattr(luogo, "street", "") and luogo.street.strip():
                if getattr(luogo, "zip_code", "") and luogo.zip_code.strip():
                    indirizzo = True

        modalita_accesso = getattr(luogo, "modalita_accesso", {})
        if not isinstance(modalita_accesso, dict):
            modalita_accesso = ""
        else:
            modalita_accesso_blocks = modalita_accesso.get("blocks", {})
            res = [x.get("text", "") for x in modalita_accesso_blocks.values()]
            if not [x for x in res if x]:
                modalita_accesso = ""

        return {
            "description": getattr(luogo, "description", "").strip(),
            "image": getattr(luogo, "image", None),
            "indirizzo": indirizzo,
            "modalita_accesso": modalita_accesso,
            "contact_info": getattr(luogo, "contact_info", None),
        }

    def plone2volto(self, url):
        portal_url = api.portal.get().absolute_url()
        frontend_domain = api.portal.get_registry_record(
            "volto.frontend_domain", default=""
        )
        if frontend_domain and url.startswith(portal_url):
            return url.replace(portal_url, frontend_domain, 1)
        return url

    def get_luoghi(self):
        if self.is_anonymous():
            return []
        pc = api.portal.get_tool("portal_catalog")

        # show_inactive ha sempre avuto una gestione... particolare! aggiungo ai
        # kw effectiveRange = DateTime() che è quello che fa Products.CMFPlone
        # nel CatalogTool.py
        query = {
            "portal_type": "Venue",
            "review_state": "published",
        }
        brains = pc(query, **{"effectiveRange": DateTime()})
        results = {}
        for brain in brains:
            luogo = brain.getObject()

            information_dict = self.information_dict(luogo)
            if all(information_dict.values()):
                continue

            parent = luogo.aq_inner.aq_parent
            if parent.title not in results:
                results[parent.title] = {
                    "url": self.plone2volto(parent.absolute_url()),
                    "children": [],
                }
            results[parent.title]["children"].append(
                {
                    "title": luogo.title,
                    "description": information_dict.get("description") and FLAG or "",
                    "url": self.plone2volto(luogo.absolute_url()),
                    "data": {
                        "image": information_dict.get("image") and FLAG or "",
                        "indirizzo": information_dict.get("indirizzo") and FLAG or "",
                        "modalita_accesso": information_dict.get("modalita_accesso")
                        and FLAG
                        or "",
                        "contact_info": information_dict.get("contact_info")
                        and FLAG
                        or "",
                    },
                }
            )

        results = dict(sorted(results.items()))
        for key in results:
            results[key]["children"].sort(key=lambda x: x["title"])

        return results


class DownloadCheckLuoghi(CheckLuoghi):
    CT = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"

    def __call__(self):
        HEADER = [
            "Titolo",
            "Descrizione",
            "Immagine in evidenza",
            "Indirizzo",
            "Modalità do accesso",
            "Contatti",
        ]

        EMPTY_ROW = [""] * 3

        luoghi = self.get_luoghi()

        workbook = Workbook()
        sheet = workbook.active
        sheet.title = "Check Persone"
        header_font = Font(bold=True)
        section_link_font = Font(underline="single", color="0563C1", size=14)
        link_fill = PatternFill(fill_type="solid", fgColor="DDDDDD")
        link_font = Font(underline="single", color="0563C1")
        section_fill = PatternFill(fill_type="solid", fgColor="E9E9E9")
        alignment = Alignment(horizontal="center", vertical="top", wrapText=True)

        section_row_height = int(14 * 1.5)

        for category, category_data in luoghi.items():
            section_url = category_data["url"]
            section_title = category
            section_row = [section_title, "", ""]
            sheet.append(section_row)
            section_cell = sheet.cell(row=sheet.max_row, column=1)

            section_cell.alignment = alignment
            section_cell.hyperlink = section_url
            sheet.merge_cells(
                start_row=sheet.max_row,
                start_column=1,
                end_row=sheet.max_row,
                end_column=3,
            )  # noqa
            for row in sheet.iter_rows(
                min_row=sheet.max_row, max_row=sheet.max_row, min_col=1, max_col=3
            ):  # noqa
                sheet.row_dimensions[row[0].row].height = section_row_height
                for cell in row:
                    cell.fill = section_fill
                    cell.font = section_link_font

            sheet.append(HEADER)
            for col in range(1, len(HEADER) + 1):
                header_cell = sheet.cell(row=sheet.max_row, column=col)
                header_cell.fill = link_fill
                header_cell.font = header_font

            for col in sheet.columns:
                column_letter = get_column_letter(col[0].column)
                sheet.column_dimensions[column_letter].width = 35

            for luogo in category_data["children"]:
                title_url = luogo["url"]
                dati_luogo = [
                    luogo["title"],
                    "X" if luogo["description"] else "",
                    "X" if luogo["data"]["image"] else "",
                    "X" if luogo["data"]["indirizzo"] else "",
                    "X" if luogo["data"]["modalita_accesso"] else "",
                    "X" if luogo["data"]["contact_info"] else "",
                ]
                row = dati_luogo
                sheet.append(row)

                title_cell = sheet.cell(row=sheet.max_row, column=1)
                check_cell = sheet.cell(row=sheet.max_row, column=2)
                check_cell.alignment = check_cell.alignment.copy(horizontal="center")
                title_cell.hyperlink = title_url
                title_cell.font = link_font
                column_letter_unit = get_column_letter(title_cell.column)
                sheet.column_dimensions[column_letter_unit].width = 60

            sheet.append(EMPTY_ROW)
            sheet.append(EMPTY_ROW)

        bytes_io = io.BytesIO()
        workbook.save(bytes_io)
        data = bytes_io.getvalue()
        self.request.response.setHeader("Content-Length", len(data))
        self.request.RESPONSE.setHeader("Content-Type", self.CT)
        self.request.response.setHeader(
            "Content-Disposition",
            "attachment; filename=check_luoghi.xlsx",
        )
        return data



# -*- coding: utf-8 -*-
from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.formwidget.geolocation import Geolocation
from plone.restapi.testing import RelativeSession
from transaction import commit


import unittest


class TestGeolocationFieldConverter(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

    def tearDown(self):
        self.api_session.close()

    def test_geolocation_null_if_not_set(self):
        api.content.create(container=self.portal, type="Venue", title="venue1")
        commit()

        response = self.api_session.get("/venue1").json()
        self.assertIsNone(response["geolocation"])

    def test_geolocation_null_if_set_to_zero(self):
        api.content.create(
            container=self.portal,
            type="Venue",
            title="venue1",
            geolocation=Geolocation(0.0, 0.0),
        )
        commit()

        response = self.api_session.get("/venue1").json()
        self.assertIsNone(response["geolocation"])

    def test_geolocation_with_value_if_set(self):
        api.content.create(
            container=self.portal,
            type="Venue",
            title="venue1",
            geolocation=Geolocation(2.0, 2.0),
        )
        commit()

        response = self.api_session.get("/venue1").json()
        self.assertEqual(response["geolocation"], {"latitude": 2.0, "longitude": 2.0})



from design.plone.contenttypes.controlpanels.settings import IDesignPloneSettings
from design.plone.contenttypes.interfaces import IDesignPloneContenttypesLayer
from plone import api
from plone.restapi.interfaces import IExpandableElement
from plone.restapi.services import Service
from plone.restapi.services.navigation.get import Navigation as BaseNavigation
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


@implementer(IExpandableElement)
@adapter(Interface, IDesignPloneContenttypesLayer)
class Navigation(BaseNavigation):
    def __call__(self, expand=False):
        result = super().__call__(expand=expand)
        try:
            show_dynamic_folders_in_footer = api.portal.get_registry_record(
                "show_dynamic_folders_in_footer",
                interface=IDesignPloneSettings,
                default=True,
            )
        except KeyError:
            show_dynamic_folders_in_footer = True
        result["navigation"]["show_in_footer"] = show_dynamic_folders_in_footer
        return result


class NavigationGet(Service):
    def reply(self):
        navigation = Navigation(self.context, self.request)
        return navigation(expand=True)["navigation"]



# -*- coding: utf-8 -*-
from DateTime import DateTime
from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.formwidget.geolocation import Geolocation
from plone.restapi.testing import RelativeSession
from Products.CMFPlone.utils import getToolByName
from transaction import commit
from uuid import uuid4
from z3c.relationfield import RelationValue
from zope.component import getUtility
from zope.intid.interfaces import IIntIds
from zope.lifecycleevent import ObjectModifiedEvent
from zope.event import notify

import unittest


class TestLuogoSchema(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

    def tearDown(self):
        self.api_session.close()

    def test_behaviors_enabled_for_luogo(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            portal_types["Venue"].behaviors,
            (
                "plone.app.content.interfaces.INameFromTitle",
                "plone.app.dexterity.behaviors.id.IShortName",
                "plone.app.dexterity.behaviors.metadata.IBasic",
                "plone.app.dexterity.behaviors.metadata.ICategorization",
                "plone.excludefromnavigation",
                "plone.relateditems",
                "plone.leadimage",
                "volto.preview_image",
                "design.plone.contenttypes.behavior.contatti_venue",
                "design.plone.contenttypes.behavior.luogo",
                "design.plone.contenttypes.behavior.argomenti",
                "design.plone.contenttypes.behavior.address_venue",
                "design.plone.contenttypes.behavior.geolocation_venue",
                "design.plone.contenttypes.behavior.additional_help_infos",
                "design.plone.contenttypes.behavior.luoghi_correlati",
                "plone.textindexer",
                "plone.translatable",
                "kitconcept.seo",
                "plone.versioning",
                "collective.taxonomy.generated.tipologia_luogo",
            ),
        )

    def test_luogo_ct_title(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual("Luogo", portal_types["Venue"].title)

    def test_luogo_fieldsets(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Venue").json()
        self.assertEqual(len(resp["fieldsets"]), 11)
        self.assertEqual(
            [x.get("id") for x in resp["fieldsets"]],
            [
                "default",
                "descrizione",
                "accesso",
                "dove",
                "orari",
                "contatti",
                "informazioni",
                "settings",
                "correlati",
                "categorization",
                "seo",
            ],
        )

    def test_luogo_required_fields(self):
        resp = self.api_session.get("@types/Venue").json()
        self.assertEqual(
            sorted(resp["required"]),
            sorted(
                [
                    "title",
                    "contact_info",
                    "modalita_accesso",
                    "description",
                    "street",
                    "city",
                    "zip_code",
                    "geolocation",
                ]
            ),
        )

    def test_luogo_fields_default_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Venue").json()
        self.assertEqual(
            resp["fieldsets"][0]["fields"],
            [
                "title",
                "description",
                "image",
                "image_caption",
                "preview_image",
                "preview_caption",
                "nome_alternativo",
                "tassonomia_argomenti",
                "luoghi_correlati",
                "tipologia_luogo",
            ],
        )

    def test_luogo_fields_descrizione_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Venue").json()
        self.assertEqual(
            resp["fieldsets"][1]["fields"],
            ["descrizione_completa", "elementi_di_interesse"],
        )

    def test_luogo_fields_accesso_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Venue").json()
        self.assertEqual(
            resp["fieldsets"][2]["fields"],
            ["modalita_accesso"],
        )

    def test_luogo_fields_dove_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Venue").json()
        self.assertEqual(
            resp["fieldsets"][3]["fields"],
            [
                "street",
                "zip_code",
                "city",
                "quartiere",
                "circoscrizione",
                "country",
                "geolocation",
                "notes",
            ],
        )

    def test_luogo_fields_orari_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Venue").json()
        self.assertEqual(
            resp["fieldsets"][4]["fields"],
            ["orario_pubblico"],
        )

    def test_luogo_fields_contatti_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Venue").json()
        self.assertEqual(
            resp["fieldsets"][5]["fields"],
            [
                "contact_info",
                "struttura_responsabile_correlati",
                "struttura_responsabile",
            ],
        )

    def test_luogo_fields_informazioni_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Venue").json()
        self.assertEqual(
            resp["fieldsets"][6]["fields"],
            [
                "ulteriori_informazioni",
            ],
        )

    def test_luogo_fields_settings_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Venue").json()
        self.assertEqual(
            resp["fieldsets"][7]["fields"],
            [
                "id",
                "exclude_from_nav",
                "versioning_enabled",
                "changeNote",
            ],
        )

    def test_luogo_fields_correlati_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Venue").json()
        self.assertEqual(
            resp["fieldsets"][8]["fields"],
            ["correlato_in_evidenza"],
        )

    def test_luogo_fields_categorization_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Venue").json()
        self.assertEqual(
            resp["fieldsets"][9]["fields"],
            # ["subjects", "language", "identificativo_mibac"] BBB dovrebbe essere così
            # ma nei test esce così perché non viene vista la patch di SchemaTweaks
            ["subjects", "language", "relatedItems", "identificativo_mibac"],
        )

    def test_luogo_fields_seo_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Venue").json()
        self.assertEqual(
            resp["fieldsets"][10]["fields"],
            [
                "seo_title",
                "seo_description",
                "seo_noindex",
                "seo_canonical_url",
                "opengraph_title",
                "opengraph_description",
                "opengraph_image",
            ],
        )


class TestLuogoApi(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

        self.news = api.content.create(
            container=self.portal, type="News Item", title="TestNews"
        )
        self.news_published = api.content.create(
            container=self.portal, type="News Item", title="TestNews published"
        )
        self.service = api.content.create(
            container=self.portal, type="Servizio", title="TestService"
        )
        self.uo = api.content.create(
            container=self.portal, type="UnitaOrganizzativa", title="TestUO"
        )

        self.venue = api.content.create(
            container=self.portal, type="Venue", title="TestVenue"
        )

        api.content.transition(obj=self.news_published, transition="publish")
        self.news_published.setEffectiveDate(DateTime())
        self.news_published.reindexObject()

        intids = getUtility(IIntIds)

        venue = RelationValue(intids.getId(self.venue))
        self.news.luoghi_correlati = [venue]
        self.news_published.luoghi_correlati = [venue]
        self.service.dove_rivolgersi = [venue]
        self.uo.luoghi_correlati = [venue]
        pcatalog = getToolByName(self.portal, "portal_catalog")
        pcatalog.manage_reindexIndex(ids=["service_venue", "uo_location", "news_venue"])
        commit()

    def tearDown(self):
        self.api_session.close()

    def test_venus_addable_types(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            ("Folder", "Link", "Image", "File"),
            portal_types["Venue"].allowed_content_types,
        )

    def test_venue_geolocation_deserializer_wrong_structure(self):
        venue = api.content.create(
            container=self.portal, type="Venue", title="Example venue"
        )

        commit()
        self.assertEqual(venue.geolocation, None)

        response = self.api_session.patch(
            venue.absolute_url(),
            json={"@type": "Venue", "title": "Foo", "geolocation": {"foo": "bar"}},
        )
        # message = response.json()["message"]

        self.assertEqual(400, response.status_code)
        # TODO: anzichè `invalid geolocation data` ritorna
        #       `Il campo geolocation è obbligatorio`
        # self.assertIn("Invalid geolocation data", message)
        # TODO: i dati vanno verificati con una chiamata alla api_session
        # self.assertEqual(venue.geolocation, None)

    def test_venue_geolocation_deserializer_right_structure(self):
        venue = api.content.create(
            container=self.portal, type="Venue", title="Example venue"
        )

        commit()
        self.assertEqual(venue.geolocation, None)

        response = self.api_session.patch(
            venue.absolute_url(),
            json={
                "@type": "Venue",
                "title": "Foo",
                "geolocation": {"latitude": 11.0, "longitude": 10.0},
            },
        )

        self.assertEqual(response.status_code, 400)
        self.assertEqual(
            response.json()["message"],
            "[{'error': 'ValidationError', "
            "'message': 'Il campo modalita_accesso è obbligatorio'}]",
        )

        text_uuid = str(uuid4())
        response = self.api_session.patch(
            venue.absolute_url(),
            json={
                "@type": "Venue",
                "title": "Foo",
                "geolocation": {"latitude": 11.0, "longitude": 10.0},
                "modalita_accesso": {
                    "blocks": {
                        text_uuid: {
                            "@type": "text",
                            "text": {"blocks": [{"text": "Test", "type": "paragraph"}]},
                        }
                    },
                    "blocks_layout": {"items": [text_uuid]},
                },
            },
        )
        self.assertEqual(response.status_code, 204)
        # TODO: i dati vanno verificati con una chiamata alla api_session
        # self.assertEqual(venue.geolocation.longitude, 10.0)
        # self.assert ... (venue.modalita_accesso)

    def test_venue_services(self):
        response = self.api_session.get(self.venue.absolute_url() + "?fullobjects")
        self.assertTrue(
            response.json()["venue_services"][0]["@id"],
            self.service.absolute_url(),
        )

    def test_venue_news(self):
        response = self.api_session.get(self.venue.absolute_url() + "?fullobjects")
        res = response.json()

        self.assertEqual(len(res["related_news"]), 2)
        self.assertEqual(
            res["related_news"][0]["@id"], self.news_published.absolute_url()
        )
        self.assertEqual(
            res["related_news"][1]["@id"],
            self.news.absolute_url(),
        )

    def test_venue_serializers(self):
        venue1 = api.content.create(container=self.portal, type="Venue", title="venue1")
        venue1.geolocation = Geolocation(44.35, 11.70)
        venue2 = api.content.create(container=self.portal, type="Venue", title="venue2")
        venue2.geolocation = Geolocation(44.35, 11.70)
        intids = getUtility(IIntIds)
        venue_rel = RelationValue(intids.getId(venue2))
        venue1.luoghi_correlati = [venue_rel]
        commit()
        response = self.api_session.post(
            "/@querystring-search",
            json={
                "metadata_fields": "_all",
                "query": [
                    {
                        "i": "portal_type",
                        "o": "plone.app.querystring.operation.selection.is",
                        "v": ["Venue"],
                    }
                ],
            },
        )
        items = response.json()["items"]
        for item in items:
            if item["id"] in ["venue1", "venue2"]:
                self.assertEqual(item["geolocation"]["latitude"], 44.35)
                self.assertEqual(item["geolocation"]["longitude"], 11.7)

    def test_venue_history_return_right_data(self):
        venue = api.content.create(container=self.portal, type="Venue", title="venue1")
        venue.description = "aaa"
        notify(ObjectModifiedEvent(venue))
        venue.description = "aaa bbb"
        notify(ObjectModifiedEvent(venue))
        commit()

        response0 = self.api_session.get(f"{venue.absolute_url()}/@history/0").json()
        response1 = self.api_session.get(f"{venue.absolute_url()}/@history/1").json()
        response2 = self.api_session.get(f"{venue.absolute_url()}/@history/2").json()

        self.assertEqual(response0["description"], "")
        self.assertEqual(response1["description"], "aaa")
        self.assertEqual(response2["description"], "aaa bbb")



from Acquisition import aq_inner, aq_parent


def EventModified(dx_event, event):
    parent = aq_parent(aq_inner(dx_event))
    if parent.portal_type == "Event":
        parent.reindexObject(idxs=["rassegna"])
    return



# -*- coding: utf-8 -*-
from design.plone.contenttypes import _
from plone.namedfile import field
from plone.supermodel import model
from zope import schema


class IMessaggio(model.Schema):
    """Marker interface for Messaggio"""

    data_messaggio = schema.Date(
        title=_("data_messaggio", default="Data del messaggio"),
        required=True,
    )

    # "Titolo del messaggio" e "Descrizione" vengono lasciati in titolo e
    # descrizione di Plone

    # TODO: aggiungere tassonomia delle tipologie di azioni richieste al
    # cittadino
    azioni_richieste = schema.Choice(
        title=_("azioni_richieste", default="Azioni richieste"),
        required=True,
        vocabulary="design.plone.contenttypes.Mockup",
    )

    azioni_pratica = schema.Choice(
        title=_("azioni_pratica", default="Azioni"),
        # vocabolario di riferimento sara' la tassonomia "Lista azioni pratica"
        vocabulary="design.plone.contenttypes.ListaAzioniPratica",
        required=True,
    )

    pratica_associata = schema.TextLine(
        title=_("pratica_associata", default="Pratica associata"),
        required=True,
    )

    data_scadenza_procedura = schema.Date(
        title=_(
            "data_scadenza_procedura",
            default="Data di scadenza della procedura",
        ),
        required=False,
    )

    documenti_allegati = field.NamedBlobFile(
        title=_("documenti_allegati", default="Documenti allegati"),
        required=False,
    )



# -*- coding: utf-8 -*-
from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_FUNCTIONAL_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID

import unittest


class TestEventCreation(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_bando_substructure_created(self):
        """
        Should have:
        - documenti
        - comunicazioni
        - esiti
        """
        item = api.content.create(
            container=self.portal,
            type="Bando",
            title="Test Bando",
        )

        self.assertEqual(
            list(item.keys()),
            ["documenti", "comunicazioni", "esiti"],
        )

        self.assertEqual(item["documenti"].portal_type, "Bando Folder Deepening")
        self.assertEqual(api.content.get_state(item["documenti"]), "private")

        self.assertEqual(item["comunicazioni"].portal_type, "Bando Folder Deepening")
        self.assertEqual(api.content.get_state(item["comunicazioni"]), "private")

        self.assertEqual(item["esiti"].portal_type, "Bando Folder Deepening")
        self.assertEqual(api.content.get_state(item["esiti"]), "private")

    def test_documento_substructure_created(self):
        """
        Should have:
        - multimedia
        """
        item = api.content.create(
            container=self.portal,
            type="Documento",
            title="Test",
        )

        self.assertEqual(
            list(item.keys()),
            ["multimedia"],
        )

        self.assertEqual(item["multimedia"].portal_type, "Document")
        self.assertEqual(api.content.get_state(item["multimedia"]), "private")
        self.assertEqual(item["multimedia"].constrain_types_mode, 1)
        self.assertEqual(
            item["multimedia"].locally_allowed_types,
            ("Image",),
        )
        self.assertTrue(item["multimedia"].exclude_from_search)

    def test_event_substructure_created(self):
        """
        Should have:
        - immagini
        - video
        - sponsor_evento
        - documenti
        """
        item = api.content.create(
            container=self.portal,
            type="Event",
            title="Test",
        )

        self.assertEqual(
            list(item.keys()),
            ["immagini", "video", "sponsor_evento", "documenti"],
        )

        self.assertEqual(item["immagini"].portal_type, "Document")
        self.assertEqual(api.content.get_state(item["immagini"]), "published")
        self.assertEqual(item["immagini"].constrain_types_mode, 1)
        self.assertEqual(
            item["immagini"].locally_allowed_types,
            ("Image", "Link"),
        )
        self.assertTrue(item["immagini"].exclude_from_search)

        self.assertEqual(item["video"].portal_type, "Document")
        self.assertEqual(api.content.get_state(item["video"]), "published")
        self.assertEqual(item["video"].constrain_types_mode, 1)
        self.assertEqual(
            item["video"].locally_allowed_types,
            ("Link",),
        )
        self.assertTrue(item["video"].exclude_from_search)

        self.assertEqual(item["sponsor_evento"].portal_type, "Document")
        self.assertEqual(api.content.get_state(item["sponsor_evento"]), "published")
        self.assertEqual(item["sponsor_evento"].constrain_types_mode, 1)
        self.assertEqual(
            item["sponsor_evento"].locally_allowed_types,
            ("Link",),
        )
        self.assertTrue(item["sponsor_evento"].exclude_from_search)

        self.assertEqual(item["documenti"].portal_type, "Document")
        self.assertEqual(api.content.get_state(item["documenti"]), "published")
        self.assertEqual(item["documenti"].constrain_types_mode, 1)
        self.assertEqual(item["documenti"].locally_allowed_types, ("File",))
        self.assertTrue(item["documenti"].exclude_from_search)

    def test_incarico_substructure_created(self):
        """
        Should have:
        - compensi-file
        - importi-di-viaggio-e-o-servizi
        """
        item = api.content.create(
            container=self.portal,
            type="Incarico",
            title="Test",
        )

        self.assertEqual(
            list(item.keys()),
            ["compensi-file", "importi-di-viaggio-e-o-servizi"],
        )

        self.assertEqual(item["compensi-file"].portal_type, "Document")
        self.assertEqual(api.content.get_state(item["compensi-file"]), "private")
        self.assertTrue(item["compensi-file"].exclude_from_search)

        self.assertEqual(item["importi-di-viaggio-e-o-servizi"].portal_type, "Document")
        self.assertEqual(
            api.content.get_state(item["importi-di-viaggio-e-o-servizi"]), "private"
        )
        self.assertTrue(item["importi-di-viaggio-e-o-servizi"].exclude_from_search)

    def test_news_substructure_created(self):
        """
        Should have:
        - multimedia
        - documenti allegati
        """
        item = api.content.create(
            container=self.portal,
            type="News Item",
            title="Test News",
        )

        self.assertEqual(
            list(item.keys()),
            ["multimedia", "documenti-allegati"],
        )

        self.assertEqual(item["multimedia"].portal_type, "Document")
        self.assertEqual(api.content.get_state(item["multimedia"]), "private")
        self.assertEqual(item["multimedia"].constrain_types_mode, 1)
        self.assertEqual(
            item["multimedia"].locally_allowed_types,
            ("Image", "Link"),
        )
        self.assertTrue(item["multimedia"].exclude_from_search)

        self.assertEqual(item["documenti-allegati"].portal_type, "Document")
        self.assertEqual(api.content.get_state(item["documenti-allegati"]), "private")
        self.assertEqual(item["documenti-allegati"].constrain_types_mode, 1)
        self.assertEqual(
            item["documenti-allegati"].locally_allowed_types,
            ("File", "Image"),
        )
        self.assertTrue(item["multimedia"].exclude_from_search)

    def test_venue_substructure_created(self):
        """
        Should have:
        - multimedia
        """
        item = api.content.create(
            container=self.portal,
            type="Venue",
            title="Test",
        )

        self.assertEqual(
            list(item.keys()),
            ["multimedia"],
        )

        self.assertEqual(item["multimedia"].portal_type, "Folder")
        self.assertEqual(api.content.get_state(item["multimedia"]), "published")
        self.assertEqual(item["multimedia"].constrain_types_mode, 1)
        self.assertEqual(
            item["multimedia"].locally_allowed_types,
            ("Image", "Link"),
        )
        self.assertTrue(item["multimedia"].exclude_from_search)

    def test_persona_substructure_created(self):
        """
        Should have:
        - foto-e-attivita-politica
        - curriculum-vitae
        - situazione-patrimoniale
        - dichiarazione-dei-redditi
        - spese-elettorali
        - spese-elettorali
        - variazione-situazione-patrimoniale" "altre-cariche
        - incarichi
        - altri-documenti
        - dichiarazione-insussistenza-cause-di-inconferibilita-e-incompatibilita
        - emolumenti-complessivi-percepiti-a-carico-della-finanza-pubblica
        """
        item = api.content.create(
            container=self.portal,
            type="Persona",
            title="Test",
        )

        self.assertEqual(
            list(item.keys()),
            [
                "foto-e-attivita-politica",
                "curriculum-vitae",
                "situazione-patrimoniale",
                "dichiarazione-dei-redditi",
                "spese-elettorali",
                "variazione-situazione-patrimoniale",
                "altre-cariche",
                "incarichi",
                "altri-documenti",
                "dichiarazione-insussistenza-cause-di-inconferibilita-e-incompatibilita",
                "emolumenti-complessivi-percepiti-a-carico-della-finanza-pubblica",
            ],
        )

        self.assertEqual(item["foto-e-attivita-politica"].portal_type, "Document")
        self.assertEqual(
            api.content.get_state(item["foto-e-attivita-politica"]), "private"
        )
        self.assertEqual(item["foto-e-attivita-politica"].constrain_types_mode, 1)
        self.assertEqual(
            item["foto-e-attivita-politica"].locally_allowed_types,
            ("Image",),
        )
        self.assertTrue(item["foto-e-attivita-politica"].exclude_from_search)

        self.assertEqual(item["curriculum-vitae"].portal_type, "Document")
        self.assertEqual(api.content.get_state(item["curriculum-vitae"]), "private")
        self.assertEqual(item["curriculum-vitae"].constrain_types_mode, 1)
        self.assertEqual(item["curriculum-vitae"].locally_allowed_types, ("File",))
        self.assertTrue(item["curriculum-vitae"].exclude_from_search)

        self.assertEqual(item["situazione-patrimoniale"].portal_type, "Document")
        self.assertEqual(
            api.content.get_state(item["situazione-patrimoniale"]), "private"
        )
        self.assertEqual(item["situazione-patrimoniale"].constrain_types_mode, 1)
        self.assertEqual(
            item["situazione-patrimoniale"].locally_allowed_types, ("File",)
        )
        self.assertTrue(item["situazione-patrimoniale"].exclude_from_search)

        self.assertEqual(item["dichiarazione-dei-redditi"].portal_type, "Document")
        self.assertEqual(
            api.content.get_state(item["dichiarazione-dei-redditi"]), "private"
        )
        self.assertEqual(item["dichiarazione-dei-redditi"].constrain_types_mode, 1)
        self.assertEqual(
            item["dichiarazione-dei-redditi"].locally_allowed_types, ("File",)
        )
        self.assertTrue(item["dichiarazione-dei-redditi"].exclude_from_search)

        self.assertEqual(item["spese-elettorali"].portal_type, "Document")
        self.assertEqual(api.content.get_state(item["spese-elettorali"]), "private")
        self.assertEqual(item["spese-elettorali"].constrain_types_mode, 1)
        self.assertEqual(item["spese-elettorali"].locally_allowed_types, ("File",))
        self.assertTrue(item["spese-elettorali"].exclude_from_search)

        self.assertEqual(item["curriculum-vitae"].portal_type, "Document")
        self.assertEqual(api.content.get_state(item["curriculum-vitae"]), "private")
        self.assertEqual(item["curriculum-vitae"].constrain_types_mode, 1)
        self.assertEqual(item["curriculum-vitae"].locally_allowed_types, ("File",))
        self.assertTrue(item["curriculum-vitae"].exclude_from_search)

        self.assertEqual(
            item["variazione-situazione-patrimoniale"].portal_type, "Document"
        )
        self.assertEqual(
            api.content.get_state(item["variazione-situazione-patrimoniale"]), "private"
        )
        self.assertEqual(
            item["variazione-situazione-patrimoniale"].constrain_types_mode, 1
        )
        self.assertEqual(
            item["variazione-situazione-patrimoniale"].locally_allowed_types, ("File",)
        )
        self.assertTrue(item["variazione-situazione-patrimoniale"].exclude_from_search)

        self.assertEqual(item["altre-cariche"].portal_type, "Document")
        self.assertEqual(api.content.get_state(item["altre-cariche"]), "private")
        self.assertEqual(item["altre-cariche"].constrain_types_mode, 1)
        self.assertEqual(item["altre-cariche"].locally_allowed_types, ("File",))
        self.assertTrue(item["altre-cariche"].exclude_from_search)

        self.assertEqual(item["incarichi"].portal_type, "Document")
        self.assertEqual(api.content.get_state(item["incarichi"]), "private")
        self.assertEqual(item["incarichi"].constrain_types_mode, 1)
        self.assertEqual(item["incarichi"].locally_allowed_types, ("Incarico",))
        self.assertTrue(item["incarichi"].exclude_from_search)

        self.assertEqual(item["altri-documenti"].portal_type, "Document")
        self.assertEqual(api.content.get_state(item["altri-documenti"]), "private")
        self.assertEqual(item["altri-documenti"].constrain_types_mode, 1)
        self.assertIn("File", item["altri-documenti"].locally_allowed_types)
        self.assertIn("Link", item["altri-documenti"].locally_allowed_types)
        self.assertIn("Image", item["altri-documenti"].locally_allowed_types)
        self.assertTrue(item["altri-documenti"].exclude_from_search)

    def test_servizio_substructure_created(self):
        """
        Should have:
        - modulistica
        - allegati
        """
        item = api.content.create(
            container=self.portal,
            type="Servizio",
            title="Test",
        )

        self.assertEqual(
            list(item.keys()),
            ["modulistica", "allegati"],
        )

        self.assertEqual(item["modulistica"].portal_type, "Document")
        self.assertEqual(api.content.get_state(item["modulistica"]), "private")
        self.assertEqual(item["modulistica"].constrain_types_mode, 1)
        self.assertEqual(item["modulistica"].locally_allowed_types, ("Link",))
        self.assertTrue(item["modulistica"].exclude_from_search)

        self.assertEqual(item["allegati"].portal_type, "Document")
        self.assertEqual(api.content.get_state(item["allegati"]), "private")
        self.assertEqual(item["allegati"].constrain_types_mode, 1)
        self.assertEqual(item["allegati"].locally_allowed_types, ("File", "Link"))
        self.assertTrue(item["allegati"].exclude_from_search)

    def test_uo_substructure_created(self):
        """
        Should have:
        - allegati
        """
        item = api.content.create(
            container=self.portal,
            type="UnitaOrganizzativa",
            title="Test",
        )

        self.assertEqual(
            list(item.keys()),
            ["allegati"],
        )

        self.assertEqual(item["allegati"].portal_type, "Document")
        self.assertEqual(api.content.get_state(item["allegati"]), "private")
        self.assertEqual(item["allegati"].constrain_types_mode, 1)
        self.assertEqual(item["allegati"].locally_allowed_types, ("File",))
        self.assertTrue(item["allegati"].exclude_from_search)



# -*- coding: utf-8 -*-
from .upgrades import logger
from Acquisition import aq_base
from plone import api
from plone.namedfile import file


def to_7300(context):
    mapping = {
        # portal_type
        "Documento Personale": {
            # field: "type"
            "immagine": "image",
            "pratica_associata": "file",
        },
        "Messaggio": {
            "documenti_allegati": "file",
        },
        "Persona": {
            "foto_persona": "image",
        },
        "RicevutaPagamento": {
            "stampa_ricevuta": "file",
            "pratica_associata": "file",
            "allegato": "file",
        },
    }

    mapping_types = {
        "image": (file.NamedImage, file.NamedBlobImage),
        "file": (file.NamedFile, file.NamedBlobFile),
    }

    for portal_type, fields in mapping.items():
        brains = api.content.find(unrestricted=True, portal_type=portal_type)
        logger.info("Updating fields for %s %s objects", portal_type, len(brains))
        for brain in brains:
            obj = aq_base(brain.getObject())
            for fieldname, _type in fields.items():
                value = getattr(obj, fieldname, None)
                # if value:
                #     import pdb; pdb.set_trace()
                if value and isinstance(value, mapping_types[_type][0]):
                    logger.info("Updated %s for %s", fieldname, brain.getPath())
                    setattr(
                        obj,
                        fieldname,
                        mapping_types[_type][1](
                            data=value.data,
                            contentType=value.contentType,
                            filename=value.filename,
                        ),
                    )
    logger.info("Finished updating fields")



"""
This is a customization for original view. in design.plone.contenttypes
we allow folder deepening to contain also Modulo CT and we need to
handle it properly
"""

from design.plone.contenttypes.behaviors.multi_file import IMultiFileSchema
from plone.restapi.interfaces import IFieldSerializer
from redturtle.bandi.browser.bando import BandoView as BaseBandoView
from redturtle.bandi.browser.bando import IBandoView
from zope.component import queryMultiAdapter
from zope.interface import implementer

MODULO_FIELDS = [
    "file_principale",
    "formato_alternativo_1",
    "formato_alternativo_2",
]


@implementer(IBandoView)
class BandoView(BaseBandoView):

    def type_hook_modulo(self, brain):
        """
        Custom data for modulo
        """
        obj = brain.getObject()
        data = {}
        for field in MODULO_FIELDS:
            field_obj = IMultiFileSchema[field]
            serializer = queryMultiAdapter(
                (field_obj, obj, self.request), IFieldSerializer
            )
            data[field] = serializer()
        return data



# -*- coding: utf-8 -*-s
from plone import api
from Products.CMFPlone.utils import safe_unicode
from uuid import uuid4

import logging
import lxml
import os
import re
import requests


logger = logging.getLogger(__name__)

draftjs_converter = os.environ.get("DRAFTJS_CONVERTER_URL")

RESOLVEUID_RE = re.compile(
    r"""(['"]resolveuid/)(.*?)(['"])""", re.IGNORECASE | re.DOTALL
)


def _fix_headers(html):
    document = lxml.html.fromstring(html)

    # https://codepen.io/tomhodgins/pen/ybgMpN
    selector = '//*[substring-after(name(), "h") >= 4]'
    for header in document.xpath(selector):
        header.tag = "h3"
    if document.tag != "div":
        return lxml.html.tostring(document)
    return "".join(safe_unicode(lxml.html.tostring(c)) for c in document.iterchildren())


def _fix_html(html):
    # cleanup html
    portal_transforms = api.portal.get_tool(name="portal_transforms")
    data = portal_transforms.convertTo("text/x-html-safe", html, mimetype="text/html")
    html = data.getData()

    if html is None:
        return ""
    document = lxml.html.fromstring(html)
    root = document
    if root.tag != "div":
        root = root.getparent()
    if root is None:
        return ""
    _extract_img_from_tags(document=document, root=root)
    _remove_empty_tags(root=root)
    return "".join(safe_unicode(lxml.html.tostring(c)) for c in root.iterchildren())


def _remove_empty_tags(root):
    if root is None:
        return
    if root.tag in ["br", "img", "iframe", "embed", "video"]:
        # it's a self-closing tag
        return

    children = root.getchildren()
    if not children:
        if root.text in [None, "", "\xa0", " ", "\r\n"]:
            # empty element
            root.getparent().remove(root)
        return
    for child in children:
        _remove_empty_tags(root=child)
    if not root.getchildren():
        # root had empty children that has been removed
        root.getparent().remove(root)


def _extract_img_from_tags(document, root):
    for image in document.xpath("//img"):
        # Get the current paragraph
        paragraph = image.getparent()
        while paragraph.getparent() not in [root, None]:
            paragraph = paragraph.getparent()
        # Get the current paragraph

        # Deal with images with links
        img_parent = image.getparent()
        if img_parent.tag == "a":
            image.attrib["data-href"] = img_parent.attrib.get("href", "")
        # Deal with images with links

        # If image has a tail, insert a new span to replace it
        if image.tail:
            if img_parent != paragraph:
                img_parent.insert(
                    img_parent.index(image),
                    lxml.html.builder.SPAN(image.tail),
                )
            else:
                paragraph.insert(
                    paragraph.index(image),
                    lxml.html.builder.SPAN(image.tail),
                )
            image.tail = ""

        # move image before paragraph
        if paragraph.getparent() is None:
            root.insert(root.index(image), lxml.html.builder.P(image))
        else:
            root.insert(root.index(paragraph), lxml.html.builder.P(image))

        # clenup empty tags
        text = ""
        if img_parent.text is not None:
            text = img_parent.text.strip()
        while len(img_parent.getchildren()) == 0 and text == "":
            parent = img_parent.getparent()
            parent.remove(img_parent)
            img_parent = parent
            text = ""
            if img_parent.text is not None:
                text = img_parent.text.strip()
        # clenup empty tags


def _fix_blocks(block):
    block_type = block.get("@type", "")
    if block_type == "text":
        entity_map = block.get("text", {}).get("entityMap", {})
        for entity in entity_map.values():
            if entity.get("type") == "LINK":
                # draftjs set link in "url" but we want handle it in "href"
                url = entity.get("data", {}).get("url", "")
                entity["data"]["href"] = url
    return block


def _conversion_tool(html):
    if not draftjs_converter:
        raise Exception(
            "DRAFTJS_CONVERTER_URL environment varialbe not set. Unable to convert html to draftjs."  # noqa
        )
    resp = requests.post(draftjs_converter, data={"html": html})
    if resp.status_code != 200:
        raise Exception("Unable to convert to draftjs this html: {}".format(html))
    return resp.json()["data"]


def to_draftjs(text):
    """
    do something here
    """
    if not text:
        return {"blocks": {}, "blocks_layout": {"items": []}}
    html = _fix_headers(text)
    html = _fix_html(html)

    blocks = {}
    blocks_layout = {"items": []}

    result = _conversion_tool(html)
    for block in result:
        block = _fix_blocks(block)
        text_uuid = str(uuid4())
        blocks[text_uuid] = block
        blocks_layout["items"].append(text_uuid)
    return {"blocks": blocks, "blocks_layout": blocks_layout}



# -*- coding: utf-8 -*-
from collective.taxonomy.interfaces import ITaxonomy
from plone import api
from Products.CMFPlone.interfaces import INonInstallable
from redturtle.bandi.interfaces.settings import IBandoSettings
from zope.component import getUtilitiesFor
from zope.interface import implementer


@implementer(INonInstallable)
class HiddenProfiles(object):
    def getNonInstallableProfiles(self):
        """Hide uninstall profile from site-creation and quickinstaller."""
        return [
            "design.plone.contenttypes:uninstall",
            "design.plone.contenttypes:to_3000",
        ]


def post_install(context):
    """Post install script"""

    remove_blocks_behavior(context)

    # update behaviors
    portal_types = api.portal.get_tool(name="portal_types")
    BEHAVIORS = {
        "Document": {
            "in": [
                "plone.leadimage",
                "volto.preview_image",
            ],
            "out": [
                "plone.leadimage",
                "volto.preview_image",
                "plone.tableofcontents",
            ],
        },
    }
    for ct in BEHAVIORS.keys():
        ct_behaviors = [
            x
            for x in portal_types[ct].behaviors
            if x not in BEHAVIORS[ct]["out"]  # noqa
        ]
        ct_behaviors.extend([x for x in BEHAVIORS[ct]["in"] if x not in ct_behaviors])
        portal_types[ct].behaviors = tuple(ct_behaviors)

    # remove default ente
    api.portal.set_registry_record("default_ente", (), interface=IBandoSettings)


def post_install_taxonomy(context):
    context.runImportStepFromProfile(
        "profile-design.plone.contenttypes:default", "typeinfo", True
    )
    # C'è una versione di collective.taxonomies in cui quel campo non viene
    # settato correttamente.
    for utility_name, utility in list(getUtilitiesFor(ITaxonomy)):
        utility.updateBehavior(**{"field_prefix": ""})


def uninstall(context):
    """Uninstall script"""
    # Do something at the end of the uninstallation of this package.


def remove_blocks_behavior(context):
    portal_types = api.portal.get_tool(name="portal_types")
    for ptype in ["News Item", "Event"]:
        portal_types[ptype].behaviors = tuple(
            [x for x in portal_types[ptype].behaviors if x != "volto.blocks"]
        )



# -*- coding: utf-8 -*-
from design.plone.contenttypes import _
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


# TODO: merge with NEWS
class ILuoghiCorrelatiSchema(model.Schema):
    luoghi_correlati = RelationList(
        title=_("luoghi_correlati_label", default="Luoghi correlati"),
        description=_(
            "luoghi_correlati_help",
            default="Seleziona una lista di luoghi citati.",
        ),
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=False,
    )
    form.widget(
        "luoghi_correlati",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["Venue"],
        },
    )


@provider(IFormFieldProvider)
class ILuoghiCorrelati(ILuoghiCorrelatiSchema):
    """
    Default fieldset
    """


@provider(IFormFieldProvider)
class ILuoghiCorrelatiEvento(model.Schema):
    """
    Events have a differente fieldset for this field
    """

    luoghi_correlati = RelationList(
        title=_("luoghi_correlati_label", default="Luoghi correlati"),
        description=_(
            "luoghi_correlati_event_help",
            default="Seleziona una lista di luoghi citati. Se il luogo "
            "dell'evento non è presente sul sito, inserisci le sue "
            "informazioni nei campi seguenti.",
        ),
        default=[],
        value_type=RelationChoice(vocabulary="plone.app.vocabularies.Catalog"),
        required=False,
    )
    form.widget(
        "luoghi_correlati",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["Venue"],
        },
        frontendOptions={
            "widget": "luoghi_correlati_evento",
        },
    )

    model.fieldset(
        "luogo",
        label=_("luogo_label", default="Luogo"),
        fields=["luoghi_correlati"],
    )


@implementer(ILuoghiCorrelati)
@adapter(IDexterityContent)
class LuoghiCorrelati(object):
    """ """

    def __init__(self, context):
        self.context = context


@implementer(ILuoghiCorrelatiEvento)
@adapter(IDexterityContent)
class LuoghiCorrelatiEvento(object):
    """ """

    def __init__(self, context):
        self.context = context



# -*- coding: utf-8 -*-

"""Setup tests for this package."""
from design.plone.contenttypes.controlpanels.settings import IDesignPloneSettings
from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import RelativeSession

import transaction
import unittest


class TestShowModifiedBehavior(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING
    maxDiff = None

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

    def tearDown(self):
        self.api_session.close()
        # reset do default
        api.portal.set_registry_record(
            "show_modified_default",
            False,
            interface=IDesignPloneSettings,
        )
        transaction.commit()

    def test_if_not_set_return_site_default(self):
        page = api.content.create(
            container=self.portal,
            type="Document",
            title="Test document",
        )
        transaction.commit()
        resp = self.api_session.get(page.absolute_url())

        self.assertTrue(getattr(page, "show_modified", None))
        self.assertTrue(resp.json().get("show_modified", None))

        api.portal.set_registry_record(
            "show_modified_default",
            False,
            interface=IDesignPloneSettings,
        )
        transaction.commit()

        resp = self.api_session.get(page.absolute_url())
        self.assertFalse(getattr(page, "show_modified", None))
        self.assertFalse(resp.json().get("show_modified", None))

    def test_if_set_will_override_default(self):
        page = api.content.create(
            container=self.portal,
            type="Document",
            title="Test document",
            show_modified=False,
        )
        transaction.commit()
        resp = self.api_session.get(page.absolute_url())

        self.assertFalse(getattr(page, "show_modified", None))
        self.assertFalse(resp.json().get("show_modified", None))



# -*- coding: utf-8 -*-
from plone.dexterity.interfaces import IDexterityContent
from plone.indexer.decorator import indexer


@indexer(IDexterityContent)
def tassonomia_argomenti(context, **kw):
    return [
        x.to_object.Title()
        for x in getattr(context.aq_base, "tassonomia_argomenti", [])
        if x.to_object
    ]


@indexer(IDexterityContent)
def tassonomia_argomenti_uid(context, **kw):
    return [
        x.to_object.UID()
        for x in getattr(context.aq_base, "tassonomia_argomenti", [])
        if x.to_object
    ]


@indexer(IDexterityContent)
def ufficio_responsabile(context, **kw):
    uffici = getattr(context.aq_base, "ufficio_responsabile", [])
    return [ufficio.UID() for ufficio in filter(bool, [x.to_object for x in uffici])]


@indexer(IDexterityContent)
def parent(context):
    obj_parent = context.aq_parent
    return {
        "title": obj_parent.Title(),
        "UID": obj_parent.UID(),
        "@id": obj_parent.absolute_url(),
        "@type": obj_parent.portal_type,
    }


@indexer(IDexterityContent)
def exclude_from_search(context):
    return getattr(context.aq_base, "exclude_from_search", False)



# -*- coding: utf-8 -*-

from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING,
)
from plone import api
from plone.app.testing import helpers
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.testing import RelativeSession
from transaction import commit
from z3c.relationfield import RelationValue
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.intid.interfaces import IIntIds

import unittest


class TestPersonaSchema(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        self.persona = api.content.create(
            container=self.portal, type="Persona", title="John Doe"
        )
        intids = getUtility(IIntIds)

        self.persona_ref = RelationValue(intids.getId(self.persona))
        commit()

    def tearDown(self):
        self.api_session.close()

    def test_behaviors_enabled_for_persona(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            portal_types["Persona"].behaviors,
            (
                "plone.namefromtitle",
                "plone.allowdiscussion",
                "plone.excludefromnavigation",
                "plone.shortname",
                "plone.ownership",
                "plone.publication",
                "plone.relateditems",
                "plone.categorization",
                "plone.basic",
                "plone.locking",
                "design.plone.contenttypes.behavior.additional_help_infos",
                "design.plone.contenttypes.behavior.contatti_persona",
                "plone.textindexer",
                "plone.translatable",
                "kitconcept.seo",
                "plone.versioning",
                "plone.constraintypes",
            ),
        )

    def test_persona_ct_title(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual("Persona pubblica", portal_types["Persona"].title)

    def test_persona_fieldsets(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Persona").json()
        self.assertEqual(len(resp["fieldsets"]), 10)
        self.assertEqual(
            [x.get("id") for x in resp["fieldsets"]],
            [
                "default",
                "ruolo",
                "contatti",
                "documenti",
                "informazioni",
                # "correlati", questo non viene fuori nei test
                "categorization",
                "dates",
                "ownership",
                "settings",
                "seo",
            ],
        )

    def test_persona_required_fields(self):
        resp = self.api_session.get("@types/Persona").json()
        self.assertEqual(
            sorted(resp["required"]),
            sorted(
                [
                    "title",
                    "contact_info",
                ]
            ),
        )

    def test_persona_fields_default_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Persona").json()
        self.assertEqual(
            resp["fieldsets"][0]["fields"],
            ["title", "description", "foto_persona"],
        )

    def test_persona_fields_ruolo_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Persona").json()
        self.assertEqual(
            resp["fieldsets"][1]["fields"],
            ["incarichi_persona", "competenze", "deleghe", "biografia"],
        )

    def test_persona_fields_contatti_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Persona").json()
        self.assertEqual(resp["fieldsets"][2]["fields"], ["contact_info"])

    def test_persona_fields_documenti_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Persona").json()
        self.assertEqual(
            resp["fieldsets"][3]["fields"],
            [
                "curriculum_vitae",
                "emolumenti_a_carico_della_finanza_pubblica",
                "dichiarazioni_di_insussistenza_e_incompatibilita",
            ],
        )

    def test_persona_fields_informazioni_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Persona").json()
        self.assertEqual(
            resp["fieldsets"][4]["fields"],
            ["ulteriori_informazioni"],
        )

    def test_persona_fields_categorization_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Persona").json()
        self.assertEqual(
            resp["fieldsets"][5]["fields"], ["relatedItems", "subjects", "language"]
        )

    def test_persona_fields_dates_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Persona").json()
        self.assertEqual(resp["fieldsets"][6]["fields"], ["effective", "expires"])

    def test_persona_fields_ownership_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Persona").json()
        self.assertEqual(
            resp["fieldsets"][7]["fields"], ["creators", "contributors", "rights"]
        )

    def test_persona_fields_settings_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Persona").json()
        self.assertEqual(
            resp["fieldsets"][8]["fields"],
            [
                "allow_discussion",
                "exclude_from_nav",
                "id",
                "versioning_enabled",
                "changeNote",
            ],
        )

    def test_persona_fields_seo_fieldset(self):
        """
        Get the list from restapi
        """
        resp = self.api_session.get("@types/Persona").json()
        self.assertEqual(
            resp["fieldsets"][9]["fields"],
            [
                "seo_title",
                "seo_description",
                "seo_noindex",
                "seo_canonical_url",
                "opengraph_title",
                "opengraph_description",
                "opengraph_image",
            ],
        )

    def test_atto_di_nomina_incarico(self):
        incarico = api.content.create(
            container=self.persona.incarichi, type="Incarico", title="Sindaco"
        )
        commit()
        atto_nomina = api.content.create(
            container=self.portal, type="Documento", title="Atto di nomina"
        )
        commit()
        intids = getUtility(IIntIds)
        self.persona.incarichi_persona = [RelationValue(intids.getId(incarico))]
        incarico.atto_nomina = [RelationValue(intids.getId(atto_nomina))]
        commit()
        response = self.api_session.get(self.persona.absolute_url())
        res = response.json()
        self.assertEqual(len(res["incarichi_persona"]), 1)
        self.assertEqual(res["incarichi_persona"][0]["title"], incarico.title)
        self.assertIn("atto_di_nomina", list(res["incarichi_persona"][0].keys()))

    def test_delete_incarico_and_call_persona(self):
        """
        This test is to check that if an incarico is deleted,
        the persona endpoint respond correctly. Right now it breaks because of
        the relation to the deleted incarico in persona ct.
        """
        incarico = api.content.create(
            container=self.persona.incarichi, type="Incarico", title="Sindaco"
        )
        commit()
        intids = getUtility(IIntIds)
        self.persona.incarichi_persona = [RelationValue(intids.getId(incarico))]
        commit()

        summary = getMultiAdapter(
            (self.persona, self.request), ISerializeToJsonSummary
        )()
        self.assertTrue(len(summary["incarichi"]) > 0)

        self.persona.incarichi._delObject(incarico.getId())
        commit()
        summary = getMultiAdapter(
            (self.persona, self.request), ISerializeToJsonSummary
        )()
        # non ho incarichi, ma soprattutto non ho errori
        self.assertTrue(len(summary["incarichi"]) == 0)

    def test_unauthorized_on_subfolder(self):
        incarico = api.content.create(
            container=self.persona.incarichi, type="Incarico", title="Sindaco"
        )
        commit()
        intids = getUtility(IIntIds)
        self.persona.incarichi_persona = [RelationValue(intids.getId(incarico))]
        api.content.transition(obj=self.persona, transition="publish")
        commit()

        helpers.logout()
        # with previous bug this as anonymous user return
        # AccessControl.unauthorized.Unauthorized: You are not allowed to
        # access '_Access_inactive_portal_content_Permission' in this context
        persona_summary = getMultiAdapter(
            (self.persona, self.request), ISerializeToJsonSummary
        )()
        self.assertFalse(persona_summary["incarichi"])
        incarico_summary = getMultiAdapter(
            (self.persona.incarichi.sindaco, self.request), ISerializeToJsonSummary
        )()
        self.assertEqual(incarico_summary["compensi_file"], [])
        self.assertEqual(incarico_summary["importi_di_viaggio_e_o_servizi"], [])



# -*- coding: utf-8 -*-
from collective.taxonomy import PATH_SEPARATOR
from collective.taxonomy.interfaces import ITaxonomy
from design.plone.contenttypes.interfaces import IDesignPloneContenttypesLayer
from design.plone.contenttypes.interfaces.dataset import IDataset
from design.plone.contenttypes.interfaces.documento import IDocumento
from design.plone.contenttypes.interfaces.incarico import IIncarico
from design.plone.contenttypes.interfaces.persona import IPersona
from design.plone.contenttypes.interfaces.pratica import IPratica
from design.plone.contenttypes.interfaces.punto_di_contatto import IPuntoDiContatto
from plone import api
from plone.app.contenttypes.interfaces import IEvent
from plone.app.contenttypes.interfaces import INewsItem
from plone.base.interfaces import IImageScalesAdapter
from plone.formwidget.geolocation.geolocation import Geolocation
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.serializer.converters import json_compatible
from Products.CMFPlone.utils import safe_hasattr
from Products.ZCatalog.interfaces import ICatalogBrain
from redturtle.volto.restapi.serializer.summary import (
    DefaultJSONSummarySerializer as BaseSerializer,
)
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.globalrequest import getRequest
from zope.i18n import translate
from zope.interface import implementer
from zope.interface import Interface
from zope.schema import getFieldsInOrder

import logging
import re


RESOLVEUID_RE = re.compile(".*?/resolve[Uu]id/([^/]*)/?(.*)$")
logger = logging.getLogger(__name__)


def get_taxonomy_information(field_name, context, res):
    """
    Get the proper values for taxonomy fields
    """
    request = getRequest()
    taxonomy = getUtility(ITaxonomy, name=f"collective.taxonomy.{field_name}")
    taxonomy_voc = taxonomy.makeVocabulary(request.get("LANGUAGE"))

    # il summary di un fullobject torna un value
    # il summary di un brain torna una lista (collective.taxonomy ha motivi per
    # fare così).

    # se abbiamo il summary di un fullobject o il summary di un brain non importa
    # ritorna quello che avevi in pancia, se non avevi nulla.

    # Hai quel campo compilato? Ti trasformo come serve al frontend
    if ICatalogBrain.providedBy(context):
        fullterms = []
        # delle volte posso avere il brain senza quel dato
        if field_name not in res:
            res[field_name] = getattr(context, field_name, None) or []
        for token in res[field_name]:
            title = taxonomy_voc.inv_data.get(token, None)
            if title and title.startswith(PATH_SEPARATOR):
                title = title.replace(PATH_SEPARATOR, "", 1)
            fullterms.append({"token": token, "title": title})
        res[field_name] = fullterms
    else:
        value = getattr(context, field_name, None)

        def get_fullterms(token):
            if not token:
                return None
            title = taxonomy_voc.inv_data.get(token, None)
            if title and title.startswith(PATH_SEPARATOR):
                title = title.replace(PATH_SEPARATOR, "", 1)
            return {
                "token": token,
                "title": title,
            }

        if isinstance(value, list):
            res[field_name] = [get_fullterms(token) for token in value]
        elif isinstance(value, str):
            res[field_name] = get_fullterms(value)

    return res


def get_taxonomy_information_by_type(res, context):
    portal_type = res.get("portal_type", None) or res.get("@type")
    portal_type_mapping = {
        "News Item": ("tipologia_notizia",),
        "Event": ("tipologia_evento",),
        "Venue": ("tipologia_luogo",),
        "Dataset": (
            "temi_dataset",
            "tipologia_frequenza_aggiornamento",
            "tipologia_licenze",
        ),
        "Documento": (
            "tipologia_documenti_albopretorio",
            "tipologia_documento",
            "tipologia_licenze",
            "person_life_events",
            "business_events",
        ),
        "Pratica": ("tipologia_stati_pratica",),
        "UnitaOrganizzativa": ("tipologia_organizzazione",),
        "Incarico": ("tipologia_incarico",),
        "Servizio": ("person_life_events", "business_events"),
    }
    for field_name in portal_type_mapping.get(portal_type, []):
        get_taxonomy_information(field_name, context, res)

    return res


def extract_geolocation(context, res):
    """
    Extracts geolocation information from the provided context or res.
    """
    # Check if latitude and longitude are already present in res
    latitude = res.get("latitude", 0)
    longitude = res.get("longitude", 0)

    if latitude and longitude:
        return {"latitude": latitude, "longitude": longitude}

    # Check if context is an ICatalogBrain and has latitude and longitude
    if ICatalogBrain.providedBy(context):
        latitude = context.latitude
        longitude = context.longitude
        if latitude and longitude:
            return {"latitude": latitude, "longitude": longitude}
    else:
        # Check if context has a geolocation attribute with latitude and longitude
        geolocation = getattr(context, "geolocation", None)
        if isinstance(geolocation, Geolocation):
            latitude = geolocation.latitude
            longitude = geolocation.longitude
            if latitude and longitude:
                return {"latitude": latitude, "longitude": longitude}

    return None


@implementer(ISerializeToJsonSummary)
@adapter(Interface, IDesignPloneContenttypesLayer)
class DefaultJSONSummarySerializer(BaseSerializer):
    def __call__(self, force_all_metadata=False, force_images=False):
        res = super().__call__(force_all_metadata=force_all_metadata)
        metadata_fields = self.metadata_fields()
        if self.context.portal_type == "Persona":
            res["incarichi"] = self.get_incarichi()
        if self.context.portal_type == "Bando":
            if "tipologia_bando" not in res:
                res["tipologia_bando"] = getattr(self.context, "tipologia_bando", "")
            if "bando_state" in metadata_fields or self.show_all_metadata_fields:
                res["bando_state"] = self.get_bando_state()

            # if default set to None
            if "apertura_bando" in metadata_fields or self.show_all_metadata_fields:
                if res.get("apertura_bando") == "1100-01-01T00:00:00":
                    res["apertura_bando"] = None

        if self.context.portal_type == "Event":
            res["start"] = json_compatible(self.context.start)
            res["end"] = json_compatible(self.context.end)
            res["recurrence"] = json_compatible(self.context.recurrence)

        if "geolocation" in metadata_fields or self.show_all_metadata_fields:
            # backward compatibility for some block templates
            if "geolocation" not in res:
                res["geolocation"] = extract_geolocation(self.context, res)

        res["id"] = self.context.id
        res["UID"] = (
            self.context.UID() if callable(self.context.UID) else self.context.UID
        )

        # meta_type
        res["design_italia_meta_type"] = self.get_design_meta_type()

        # tassonomia argomenti
        if "tassonomia_argomenti" in res:
            if res["tassonomia_argomenti"]:
                res["tassonomia_argomenti"] = self.expand_tassonomia_argomenti()

        get_taxonomy_information_by_type(res, self.context)

        if self.is_get_call():
            res["has_children"] = self.has_children()
        if force_images:
            # TODO: verificare se non c'è il campo o se il campo è null/vuoto ?
            if not res.get("image_scales") and not res.get("image_field"):
                adapter = queryMultiAdapter(
                    (self.context, self.request), IImageScalesAdapter
                )
                if adapter:
                    scales = adapter()
                    if scales:
                        res["image_scales"] = scales
                    if "preview_image" in scales:
                        res["image_field"] = "preview_image"
                    elif "image" in scales:
                        res["image_field"] = "image"
            # plone.app.contentlisting 3.0.5 sembra fissare un accessor che ci
            # fa arrivare uno scale senza image_field.
            # avremmo questo problema una volta passati a plone 6.0.11
            elif res.get("image_scales") and not res.get("image_field"):
                if "preview_image" in res["image_scales"]:
                    res["image_field"] = "preview_image"
                elif "image" in res["image_scales"]:
                    res["image_field"] = "image"
        return res

    def has_children(self):
        try:
            obj = self.context.getObject()
        except AttributeError:
            obj = self.context
        try:
            if obj.aq_base.keys():
                return True
        except AttributeError:
            return False
        return False

    def is_get_call(self):
        steps = self.request.steps
        if not steps:
            return False
        return steps[-1] == "GET_application_json_"

    def get_design_meta_type(self):
        ttool = api.portal.get_tool("portal_types")
        if self.context.portal_type == "News Item":
            tipologia_notizia = getattr(self.context, "tipologia_notizia", "")
            if tipologia_notizia:
                taxonomy = getUtility(
                    ITaxonomy, name="collective.taxonomy.tipologia_notizia"
                )
                taxonomy_voc = taxonomy.makeVocabulary(self.request.get("LANGUAGE"))
                if isinstance(tipologia_notizia, list):
                    token = tipologia_notizia[0]
                else:
                    token = tipologia_notizia
                title = taxonomy_voc.inv_data.get(token, None)
                if title:
                    if title.startswith(PATH_SEPARATOR):
                        title = title.replace(PATH_SEPARATOR, "", 1)
                    return title
        if self.context.portal_type in ttool:
            return translate(
                ttool[self.context.portal_type].Title(), context=self.request
            )
        else:
            return self.context.portal_type

    def expand_tassonomia_argomenti(self):
        try:
            obj = self.context.getObject()
        except AttributeError:
            obj = self.context
        arguments = []
        for ref in getattr(obj, "tassonomia_argomenti", []):
            ref_obj = ref.to_object
            if not ref_obj:
                continue
            if not api.user.has_permission("View", obj=ref_obj):
                continue
            arguments.append(
                getMultiAdapter((ref_obj, self.request), ISerializeToJsonSummary)()
            )
        return arguments

    def get_bando_state(self):
        """
        È il metodo più safe per ottenere lo stato del bando
        anche se non il più veloce
        """
        bando = self.context.getObject()
        view = api.content.get_view("bando_view", context=bando, request=self.request)
        return view.getBandoState()

    # TODO: use tipo incarico from taxonomy when taxonomies are ready
    # instead of CT title
    def get_incarichi(self):
        try:
            obj = self.context.getObject()
        except AttributeError:
            obj = self.context

        incarichi = []
        for incarico in obj.incarichi_persona:
            if not incarico.to_object:
                continue

            if not api.user.has_permission("View", obj=incarico.to_object):
                continue
            incarichi.append(incarico.to_object.title)
        return ", ".join(incarichi)


@implementer(ISerializeToJsonSummary)
@adapter(IIncarico, IDesignPloneContenttypesLayer)
class IncaricoDefaultJSONSummarySerializer(DefaultJSONSummarySerializer):
    def __call__(self, **kwargs):
        res = super().__call__(**kwargs)
        get_taxonomy_information("tipologia_incarico", self.context, res)
        if "data_inizio_incarico" not in res:
            res["data_inizio_incarico"] = json_compatible(
                self.context.data_inizio_incarico
            )
        else:
            res["data_inizio_incarico"] = json_compatible(None)

        if "data_conclusione_incarico" not in res:
            res["data_conclusione_incarico"] = json_compatible(
                self.context.data_conclusione_incarico
            )
        else:
            res["data_conclusione_incarico"] = json_compatible(None)

        if "data_insediamento" not in res:
            res["data_insediamento"] = json_compatible(self.context.data_insediamento)
        else:
            res["data_insediamento"] = json_compatible(None)

        if "compensi" not in res:
            res["compensi"] = json_compatible(self.context.compensi)
        else:
            res["compensi"] = json_compatible([])

        if "importi_viaggio_servizio" not in res:
            res["importi_viaggio_servizio"] = json_compatible(
                self.context.importi_viaggio_servizio
            )
        else:
            res["importi_viaggio_servizio"] = json_compatible([])

        if safe_hasattr(self.context, "compensi-file"):
            compensi_folder = getattr(self.context, "compensi-file")
            res["compensi_file"] = []
            if api.user.has_permission("View", obj=compensi_folder):
                for brain in getattr(self.context, "compensi-file").getFolderContents():
                    res["compensi_file"].append(
                        getMultiAdapter(
                            (brain, self.request), ISerializeToJsonSummary
                        )()
                    )

        if safe_hasattr(self.context, "importi-di-viaggio-e-o-servizi"):
            importi_folder = getattr(self.context, "importi-di-viaggio-e-o-servizi")
            res["importi_di_viaggio_e_o_servizi"] = []
            if api.user.has_permission("View", obj=importi_folder):
                for brain in importi_folder.getFolderContents():
                    res["importi_di_viaggio_e_o_servizi"].append(
                        getMultiAdapter(
                            (brain, self.request), ISerializeToJsonSummary
                        )()
                    )

        if "atto_di_nomina" not in res:
            res["atto_di_nomina"] = None
            atto = getattr(self.context, "atto_nomina", None)
            if atto and not atto[0].isBroken():
                atto = atto[0].to_object
                res["atto_di_nomina"] = atto.absolute_url()
        return res


@implementer(ISerializeToJsonSummary)
@adapter(IPuntoDiContatto, IDesignPloneContenttypesLayer)
class PuntoDiContattoDefaultJSONSummarySerializer(DefaultJSONSummarySerializer):
    def __call__(self, **kwargs):
        res = super().__call__(**kwargs)
        res["value_punto_contatto"] = self.context.value_punto_contatto
        return res


# TODO: questo potrebbe non essere più necessario, vista l'implementazione
# di DefaultJSONSummarySerializer con image_scales e image_field
@implementer(ISerializeToJsonSummary)
@adapter(IPersona, IDesignPloneContenttypesLayer)
class PersonaDefaultJSONSummarySerializer(DefaultJSONSummarySerializer):
    def __call__(self, **kwargs):
        res = super().__call__(**kwargs)
        fields = dict(getFieldsInOrder(IPersona))
        field = fields.get("foto_persona", None)
        if field:
            images_info_adapter = getMultiAdapter(
                (field, self.context, IDesignPloneContenttypesLayer)
            )
            if images_info_adapter:
                res["image_scales"] = {
                    "foto_persona": [images_info_adapter()],
                }
            res["image_field"] = "foto_persona"
        return res


@implementer(ISerializeToJsonSummary)
@adapter(IEvent, IDesignPloneContenttypesLayer)
class EventDefaultJSONSummarySerializer(DefaultJSONSummarySerializer):
    def __call__(self, force_images=True, **kwargs):
        res = super().__call__(force_images=force_images, **kwargs)
        get_taxonomy_information("tipologia_evento", self.context, res)
        return res


@implementer(ISerializeToJsonSummary)
@adapter(INewsItem, IDesignPloneContenttypesLayer)
class NewsDefaultJSONSummarySerializer(DefaultJSONSummarySerializer):
    def __call__(self, **kwargs):
        res = super().__call__(**kwargs)
        get_taxonomy_information("tipologia_notizia", self.context, res)
        return res


@implementer(ISerializeToJsonSummary)
@adapter(IDataset, IDesignPloneContenttypesLayer)
class DatasetDefaultJSONSummarySerializer(DefaultJSONSummarySerializer):
    def __call__(self, **kwargs):
        res = super().__call__(**kwargs)
        get_taxonomy_information("temi_dataset", self.context, res)
        get_taxonomy_information("tipologia_frequenza_aggiornamento", self.context, res)
        get_taxonomy_information("tipologia_licenze", self.context, res)
        return res


@implementer(ISerializeToJsonSummary)
@adapter(IDocumento, IDesignPloneContenttypesLayer)
class DocumentoPubblicoDefaultJSONSummarySerializer(DefaultJSONSummarySerializer):
    def __call__(self, **kwargs):
        res = super().__call__(**kwargs)
        get_taxonomy_information("tipologia_documenti_albopretorio", self.context, res)
        get_taxonomy_information("tipologia_documento", self.context, res)
        get_taxonomy_information("tipologia_licenze", self.context, res)
        get_taxonomy_information("person_life_events", self.context, res)
        get_taxonomy_information("business_events", self.context, res)
        return res


@implementer(ISerializeToJsonSummary)
@adapter(IPratica, IDesignPloneContenttypesLayer)
class PraticaDefaultJSONSummarySerializer(DefaultJSONSummarySerializer):
    def __call__(self, **kwargs):
        res = super().__call__(**kwargs)
        get_taxonomy_information("tipologia_stati_pratica", self.context, res)
        return res



# -*- coding: utf-8 -*-
from plone import api
from Products.Five import BrowserView
from six import StringIO

import csv


class View(BrowserView):
    def __call__(self, *args, **kwargs):
        self.request.response.setHeader("Content-type", "application/csv")
        self.request.response.setHeader(
            "Content-dispsition", "attachment; filename=incarichi.csv"
        )

        sbuf = StringIO()

        fieldnames = ["nome", "incarico", "data inizio", "tipo", "url"]
        writer = csv.DictWriter(sbuf, fieldnames=fieldnames, quoting=csv.QUOTE_ALL)
        writer.writeheader()
        brains = api.content.find(portal_type="Persona")
        for brain in brains:
            ruolo = brain.ruolo and brain.ruolo[0] or ""
            persona = brain.getObject()
            incarico_obj = (
                persona.incarichi_persona
                and persona.incarichi_persona[0].to_object
                or None
            )
            data_inizio = incarico_obj and incarico_obj.data_inizio_incarico or ""
            if data_inizio:
                data_inizio = data_inizio.strftime("%d/%m/%Y")
            tipo_incarico = incarico_obj and incarico_obj.tipologia_incarico or ""
            writer.writerow(
                {
                    "nome": brain.Title,
                    "incarico": ruolo,
                    "data inizio": data_inizio,
                    "tipo": tipo_incarico,
                    "url": brain.getURL(),
                }
            )

        res = sbuf.getvalue()
        sbuf.close()
        return res



# -*- coding: utf-8 -*-
from collective.volto.blocksfield.field import BlocksField
from design.plone.contenttypes import _
from plone.namedfile import field
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope import schema


class IDocumentoPersonale(model.Schema):
    """Marker interface for DocumentoPersonale"""

    protocollo = schema.TextLine(
        title=_("protocollo", default="Protocollo"), required=True
    )

    data_protocollo = schema.Date(
        title=_("data_protocollo", default="Data del protocollo"),
        required=True,
    )

    immagine = field.NamedBlobImage(
        title=_("immagine", default="Immagine"), required=False
    )

    pratica_associata = field.NamedBlobFile(
        title=_("pratica_associata", default="Pratica associata"),
        required=True,
    )

    servizio_origine = schema.Choice(
        title=_("servizio_origine", default="Servizio che genera il documento"),
        vocabulary="design.plone.contenttypes.Mockup",
        required=True,
    )

    # TODO: come gestiamo i campi "Tipologia del documento", "sottotipologia
    # del documento" e "lingua del documento"?

    # TODO: il vocabolario controllato da usare sara' "Argomenti di interesse
    #  pere gli utenti di un comune"
    argomenti_utenti = schema.Choice(
        title=_("argomenti_utenti", default="Argomenti utenti"),
        vocabulary="design.plone.contenttypes.Mockup",
        required=True,
    )

    oggetto = BlocksField(
        title=_("oggetto", default="Oggetto"),
        # non viene specificato se il campo e' obbligatorio o meno
        required=False,
    )

    # TODO: decidere se "link al documento" sara' un folder o un file. Se e'
    # un folder "formati disponibili" diventa un campo
    # generato facendo una query dei file presenti all'interno di "link al
    #  documento"?
    # Inserito come folder "Allegati" per il momento, magari si chiede

    # TODO: usare vocabolario dinamico per le tipologie di uffici (dovrebbe
    # rientrare nel content type "Unita' organizzativa")
    ufficio_responsabile = schema.Choice(
        title=_(
            "ufficio_responsabile_documento_personale",
            default="Ufficio responsabile",
        ),
        required=True,
        vocabulary="design.plone.contenttypes.Mockup",
    )

    # TODO: usare vocabolario dinamico per le tipologie di aree amministrative
    # (dovrebbe rientrare nel content type "Unita' organizzativa")
    area_responsabile = schema.Choice(
        title=_(
            "area_responsabile_documento_personale",
            default="Area responsabile",
        ),
        required=True,
        vocabulary="design.plone.contenttypes.Mockup",
    )

    # TODO: usare vocabolario dinamico per le gli autori?
    autori = RelationList(
        title="Autore/i",
        default=[],
        value_type=RelationChoice(
            title=_("Autore"), vocabulary="plone.app.vocabularies.Catalog"
        ),
        required=False,
    )

    licenza_distribuzione = schema.TextLine(
        title=_("licenza_distribuzione", default="Licenza di distribuzione"),
        required=False,
    )

    # TODO: usare vocabolario dinamico per i servizi collegati?
    servizi_collegati = RelationList(
        title="Servizi collegati",
        default=[],
        value_type=RelationChoice(
            title=_("Servizio collegato"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
        required=False,
    )

    canale_digitale_servizio = schema.TextLine(
        title=_(
            "canale_digitale_servizio",
            default="Canale digitale servizio collegato",
        ),
        required=False,
    )

    data_inizio = schema.Date(
        title=_("data_inizio", default="Data di inizio"), required=False
    )

    data_e_fasi_intermedie = BlocksField(
        title=_("data_e_fasi_intermedie", default="Data e fasi intermedie"),
        required=False,
    )

    data_inizio = schema.Date(
        title=_("data_inizio", default="Data di inizio"), required=False
    )

    # TODO: vocabolario per i dataset collegati ad un documento
    dataset = RelationList(
        title=_("Dataset"),
        required=False,
        default=[],
        value_type=RelationChoice(
            title=_("Dataset collegato"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
    )

    informazioni = BlocksField(
        title=_("informazioni", default="Ulteriori informazioni"),
        required=False,
    )

    riferimenti_normativi = BlocksField(
        title=_("riferimenti_normativi", default="Riferimenti normativi"),
        required=False,
    )
